# -*- coding: utf-8;

# Do not edit, this file is generated. See generate.py.

from __future__ import unicode_literals


language_names = {
"C": {
 "aa":"Afar",
 "ab":"Abkhazian",
 "ae":"Avestan",
 "af":"Afrikaans",
 "am":"Amharic",
 "ar":"Arabic",
 "as":"Assamese",
 "ast":"Asturian",
 "ay":"Aymara",
 "az":"Azerbaijani",
 "ba":"Bashkir",
 "be":"Belarusian",
 "be@latin":"Belarusian (Latin)",
 "bg":"Bulgarian",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengali",
 "bn_IN":"Bengali (India)",
 "bo":"Tibetan",
 "br":"Breton",
 "bs":"Bosnian",
 "ca":"Catalan",
 "ca@valencia":"Catalan (Valencian)",
 "ce":"Chechen",
 "ch":"Chamorro",
 "co":"Corsican",
 "crh":"Crimean Tatar",
 "cs":"Czech",
 "csb":"Kashubian",
 "cu":"Church Slavic",
 "cv":"Chuvash",
 "cy":"Welsh",
 "da":"Danish",
 "de":"German",
 "dsb":"Lower Sorbian",
 "dz":"Dzongkha",
 "el":"Greek",
 "en":"English",
 "en_GB":"British English",
 "en_US":"American English",
 "eo":"Esperanto",
 "es":"Spanish",
 "et":"Estonian",
 "eu":"Basque",
 "fa":"Farsi (Persian)",
 "fi":"Finnish",
 "fj":"Fijian",
 "fo":"Faroese",
 "fr":"French",
 "fy":"Frisian",
 "ga":"Irish Gaelic",
 "gd":"Gaelic",
 "gl":"Galician",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Hebrew",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Croatian",
 "hsb":"Upper Sorbian",
 "hu":"Hungarian",
 "hy":"Armenian",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonesian",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Icelandic",
 "it":"Italian",
 "iu":"Inuktitut",
 "ja":"Japanese",
 "jv":"Javanese",
 "ka":"Georgian",
 "ki":"Kikuyu",
 "kk":"Kazakh",
 "kl":"Kalaallisut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Korean",
 "ks":"Kashmiri",
 "ku":"Kurdish",
 "kv":"Komi",
 "kw":"Cornish",
 "ky":"Kirghiz",
 "la":"Latin",
 "lb":"Luxembourgish",
 "li":"Limburgan",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Lithuanian",
 "lv":"Latvian",
 "mai":"Maithili",
 "mg":"Malagasy",
 "mh":"Marshallese",
 "mi":"Maori",
 "mk":"Macedonian",
 "ml":"Malayalam",
 "mn":"Mongolian",
 "mo":"Moldavian",
 "mr":"Marathi",
 "ms":"Malay",
 "mt":"Maltese",
 "my":"Burmese",
 "na":"Nauru",
 "nb":"Norwegian Bokmål",
 "nd":"Ndebele, North",
 "nds":"Low Saxon",
 "ne":"Nepali",
 "ng":"Ndonga",
 "nl":"Dutch",
 "nn":"Norwegian Nynorsk",
 "nr":"Ndebele, South",
 "nso":"Northern Sotho",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Occitan",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Ossetian",
 "pa":"Punjabi/Panjabi",
 "pi":"Pali",
 "pl":"Polish",
 "ps":"Pushto",
 "pt":"Portuguese",
 "pt_BR":"Brazilian Portuguese",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Romanian",
 "rom":"Romany",
 "ru":"Russian",
 "rw":"Kinyarwanda",
 "sa":"Sanskrit",
 "sc":"Sardinian",
 "sd":"Sindhi",
 "se":"Northern Sami",
 "sg":"Sango",
 "si":"Sinhala",
 "sk":"Slovak",
 "sl":"Slovenian",
 "sm":"Samoan",
 "sn":"Shona",
 "so":"Somali",
 "sq":"Albanian",
 "sr":"Serbian",
 "sr@ijekavian":"Serbian Ijekavian",
 "sr@ijekavianlatin":"Serbian Ijekavian Latin",
 "sr@latin":"Serbian Latin",
 "ss":"Swati",
 "st":"Sotho, Southern",
 "su":"Sundanese",
 "sv":"Swedish",
 "sw":"Swahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tajik",
 "th":"Thai",
 "ti":"Tigrinya",
 "tk":"Turkmen",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turkish",
 "ts":"Tsonga",
 "tt":"Tatar",
 "tw":"Twi",
 "ty":"Tahitian",
 "ug":"Uyghur",
 "uk":"Ukrainian",
 "ur":"Urdu",
 "uz":"Uzbek",
 "uz@cyrillic":"Uzbek (Cyrillic)",
 "ven":"Venda",
 "vi":"Vietnamese",
 "vo":"Volapük",
 "wa":"Walloon",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Yiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chinese",
 "zh_CN":"Chinese Simplified",
 "zh_HK":"Chinese (Hong Kong)",
 "zh_TW":"Chinese Traditional",
 "zu":"Zulu",
},
"cs": {
 "aa":"Afar",
 "ab":"Abcházský",
 "ae":"Avestánský",
 "af":"Afrikánský",
 "am":"Amharský",
 "ar":"Arabský",
 "as":"Asamský",
 "ast":"Asturijský",
 "ay":"Aymara",
 "az":"Ázerbajdžánský",
 "ba":"Baškirský",
 "be":"Běloruský",
 "be@latin":"Běloruský (latinka)",
 "bg":"Bulharský",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengálský",
 "bn_IN":"Bengálský (Indie)",
 "bo":"Tibetský",
 "br":"Bretonský",
 "bs":"Bosenský",
 "ca":"Katalánský",
 "ca@valencia":"Katalánský (Valencie)",
 "ce":"Čečenský",
 "ch":"Chamorro",
 "co":"Korsický",
 "crh":"Krymská tatarština",
 "cs":"Český",
 "csb":"Kašubský",
 "cu":"Staroslověnský",
 "cv":"Čuvašský",
 "cy":"Welšský",
 "da":"Dánský",
 "de":"Německý",
 "dsb":"Dolnolužicko srbský",
 "dz":"Dzongkha",
 "el":"Řecký",
 "en":"Anglický",
 "en_GB":"Britská angličtina",
 "en_US":"Americká angličtina",
 "eo":"Esperanto",
 "es":"Španělský",
 "et":"Estonský",
 "eu":"Baskický",
 "fa":"Farsi (Perský)",
 "fi":"Finský",
 "fj":"Fidžijský",
 "fo":"Faerský",
 "fr":"Francouzský",
 "fy":"Fríský",
 "ga":"Galský (irština)",
 "gd":"Galský",
 "gl":"Galicijský",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Hebrejský",
 "hi":"Hindský",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Chorvatský",
 "hsb":"Hornolužicko srbský",
 "hu":"Maďarský",
 "hy":"Arménský",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonéský",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandský",
 "it":"Italský",
 "iu":"Inuktitut",
 "ja":"Japonský",
 "jv":"Jávský",
 "ka":"Gruzínský",
 "ki":"Kikuyu",
 "kk":"Kazašský",
 "kl":"Kalaallisut",
 "km":"Kmérský",
 "kn":"Kannada",
 "ko":"Korejský",
 "ks":"Kašmírský",
 "ku":"Kurdský",
 "kv":"Komi",
 "kw":"Kornišský",
 "ky":"Kyrgyzský",
 "la":"Latina",
 "lb":"Lucemburský",
 "li":"Limburgan",
 "ln":"Lingala",
 "lo":"Laoský",
 "lt":"Litevský",
 "lv":"Lotyšský",
 "mai":"Maithili",
 "mg":"Malagasy",
 "mh":"Marshallský",
 "mi":"Maorský",
 "mk":"Makedonský",
 "ml":"Malayalam",
 "mn":"Mongolský",
 "mo":"Moldavský",
 "mr":"Marathi",
 "ms":"Malajský",
 "mt":"Maltézský",
 "my":"Barmský",
 "na":"Nauru",
 "nb":"Norský (Bokmål)",
 "nd":"Ndebele, Severní",
 "nds":"Dolnosaský",
 "ne":"Nepálský",
 "ng":"Ndonga",
 "nl":"Nizozemský",
 "nn":"Norský (Nynorsk)",
 "nr":"Ndebele, Jižní",
 "nso":"Severní Sotho",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Okcitánský",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Osetský",
 "pa":"Pandžábský",
 "pi":"Pali",
 "pl":"Polský",
 "ps":"Pushto",
 "pt":"Portugalský",
 "pt_BR":"Portugalský (Brazílie)",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Rumunský",
 "rom":"Romský",
 "ru":"Ruský",
 "rw":"Kinyarwanda",
 "sa":"Sanskrit",
 "sc":"Sardinský",
 "sd":"Sindhi",
 "se":"Severní Sami",
 "sg":"Sango",
 "si":"Sinhalský",
 "sk":"Slovenský",
 "sl":"Slovinský",
 "sm":"Samojský",
 "sn":"Shona",
 "so":"Somálský",
 "sq":"Albánský",
 "sr":"Srbský",
 "sr@ijekavian":"Srbský Ijekavský",
 "sr@ijekavianlatin":"Srbský (Ijekavská latinka)",
 "sr@latin":"Srbský (latinka)",
 "ss":"Swati",
 "st":"Sotho, Jižní",
 "su":"Sundanský",
 "sv":"Švédský",
 "sw":"Svahilský",
 "ta":"Tamilský",
 "te":"Telugu",
 "tg":"Tádžický",
 "th":"Thajský",
 "ti":"Tigrinya",
 "tk":"Turkmenský",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turecký",
 "ts":"Tsonga",
 "tt":"Tatarský",
 "tw":"Twi",
 "ty":"Tahitský",
 "ug":"Ujgurský",
 "uk":"Ukrajinský",
 "ur":"Urdu",
 "uz":"Uzbecký",
 "uz@cyrillic":"Uzbecký (azbuka)",
 "ven":"Venda",
 "vi":"Vietnamský",
 "vo":"Volapük",
 "wa":"Valonský",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Jidiš",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Čínský",
 "zh_CN":"Čínský (zjednodušená)",
 "zh_HK":"Čínský (Hong Kong)",
 "zh_TW":"Čínský (tradiční)",
 "zu":"Zulu",
},
"de": {
 "aa":"Afar",
 "ab":"Abhasisch",
 "ae":"Avestisch",
 "af":"Afrikaans",
 "am":"Amharisch",
 "ar":"Arabisch",
 "as":"Assamesisch",
 "ast":"Asturisch",
 "ay":"Aimara",
 "az":"Aserbaidschanisch",
 "ba":"Baschkirisch",
 "be":"Weißrussisch",
 "be@latin":"Weißrussisch (lat. Alphabet)",
 "bg":"Bulgarisch",
 "bh":"Biharisch",
 "bi":"Bislama",
 "bn":"Bengalisch",
 "bn_IN":"Bengalisch (Indien)",
 "bo":"Tibetisch",
 "br":"Bretonisch",
 "bs":"Bosnisch",
 "ca":"Katalanisch",
 "ca@valencia":"Katalanisch (Valenciana)",
 "ce":"Tschetschenisch",
 "ch":"Chamorro",
 "co":"Korsisch",
 "crh":"Krimtatarisch",
 "cs":"Tschechisch",
 "csb":"Kaschubisch",
 "cu":"Kirchenslawisch",
 "cv":"Tschuwaschisch",
 "cy":"Walisisch",
 "da":"Dänisch",
 "de":"Deutsch",
 "dsb":"Niedersorbisch",
 "dz":"Dzongkha",
 "el":"Griechisch",
 "en":"Englisch",
 "en_GB":"Englisch (UK)",
 "en_US":"Englisch (US)",
 "eo":"Esperanto",
 "es":"Spanisch",
 "et":"Estnisch",
 "eu":"Baskisch",
 "fa":"Farsi (Persisch)",
 "fi":"Finnisch",
 "fj":"Fidschi",
 "fo":"Färöisch",
 "fr":"Französisch",
 "fy":"Friesisch",
 "ga":"Irisches Gälisch",
 "gd":"Gälisch",
 "gl":"Galicisch",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Hebräisch",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Kroatisch",
 "hsb":"Obersorbisch",
 "hu":"Ungarisch",
 "hy":"Armenisch",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonesisch",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Isländisch",
 "it":"Italienisch",
 "iu":"Inuktitut",
 "ja":"Japanisch",
 "jv":"Javanisch",
 "ka":"Georgisch",
 "ki":"Kikuyu",
 "kk":"Kasachisch",
 "kl":"Kalaallisut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Koreanisch",
 "ks":"Kashmiri",
 "ku":"Kurdisch",
 "kv":"Komi",
 "kw":"Kornisch",
 "ky":"Kirgisisch",
 "la":"Latein",
 "lb":"Luxemburgisch",
 "li":"Limburgisch",
 "ln":"Lingala",
 "lo":"Laotisch",
 "lt":"Litauisch",
 "lv":"Lettisch",
 "mai":"Maithili",
 "mg":"Madagassisch",
 "mh":"Marshallisch",
 "mi":"Maori",
 "mk":"Mazedonisch",
 "ml":"Malayalam",
 "mn":"Mongolisch",
 "mo":"Moldawisch",
 "mr":"Marathi",
 "ms":"Malaiisch",
 "mt":"Maltesisch",
 "my":"Burmesisch",
 "na":"Nauruisch",
 "nb":"Norwegisch (Bokmål)",
 "nd":"Ndebele (nördliches)",
 "nds":"Niederdeutsch",
 "ne":"Nepali",
 "ng":"Oshivambo",
 "nl":"Niederländisch",
 "nn":"Norwegisch (Nynorsk)",
 "nr":"Ndebele (südliches)",
 "nso":"Nord-Sotho",
 "nv":"Navajo",
 "ny":"Nyanja",
 "oc":"Okzitanisch",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Ossetisch",
 "pa":"Pandschabi",
 "pi":"Pali",
 "pl":"Polnisch",
 "ps":"Paschtu",
 "pt":"Portugiesisch",
 "pt_BR":"Brasilianisches Portugiesisch",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Rumänisch",
 "rom":"Romani",
 "ru":"Russisch",
 "rw":"Kinyarwanda",
 "sa":"Sanskrit",
 "sc":"Sardisch",
 "sd":"Sindhi",
 "se":"Nördliches Sami",
 "sg":"Sango",
 "si":"Singhalesisch",
 "sk":"Slowakisch",
 "sl":"Slowenisch",
 "sm":"Samoanisch",
 "sn":"Shona",
 "so":"Somali",
 "sq":"Albanisch",
 "sr":"Serbisch",
 "sr@ijekavian":"Štokavisch",
 "sr@ijekavianlatin":"Štokavisch (lat. Alphabet)",
 "sr@latin":"Serbisch (lat. Alphabet)",
 "ss":"Swati",
 "st":"Südliches Sotho",
 "su":"Sundanesisch",
 "sv":"Schwedisch",
 "sw":"Swahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tadschikisch",
 "th":"Thailändisch",
 "ti":"Tigrinja",
 "tk":"Turkmenisch",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Türkisch",
 "ts":"Tsonga",
 "tt":"Tatarisch",
 "tw":"Twi",
 "ty":"Tahitianisch",
 "ug":"Uigurisch",
 "uk":"Ukrainisch",
 "ur":"Urdu",
 "uz":"Usbekisch",
 "uz@cyrillic":"Usbekisch (Kyrillisch)",
 "ven":"Tshivenda",
 "vi":"Vietnamesisch",
 "vo":"Volapük",
 "wa":"Wallonisch",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Jiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chinesisch",
 "zh_CN":"Chinesisch (Kurzzeichen)",
 "zh_HK":"Chinesisch (Hongkong)",
 "zh_TW":"Chinesisch (Langzeichen)",
 "zu":"Zulu",
},
"es": {
 "aa":"Afar",
 "ab":"Abjaso",
 "ae":"Avéstico",
 "af":"Afrikaans",
 "am":"Amárico",
 "ar":"Árabe",
 "as":"Asamés",
 "ast":"Asturiano",
 "ay":"Aymara",
 "az":"Azerbayano",
 "ba":"Baskir",
 "be":"Bielorruso",
 "be@latin":"Bielorruso (Latino)",
 "bg":"Búlgaro",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengalí",
 "bn_IN":"Bengalí (India)",
 "bo":"Tibetano",
 "br":"Bretón",
 "bs":"Bosnio",
 "ca":"Catalán",
 "ca@valencia":"Catalán (Valenciano)",
 "ce":"Checheno",
 "ch":"Chamorro",
 "co":"Corso",
 "crh":"Tártaro de Crimea",
 "cs":"Checo",
 "csb":"Casubio",
 "cu":"Eslavo eclesiástico",
 "cv":"Chuvasio",
 "cy":"Galés",
 "da":"Danés",
 "de":"Alemán",
 "dsb":"Bajo sorabo",
 "dz":"Dzongkha",
 "el":"Griego",
 "en":"Inglés",
 "en_GB":"Inglés británico",
 "en_US":"Inglés americano",
 "eo":"Esperanto",
 "es":"Español",
 "et":"Estonio",
 "eu":"Euskera",
 "fa":"Persa",
 "fi":"Finés",
 "fj":"Fiyiano",
 "fo":"Feroés",
 "fr":"Francés",
 "fy":"Frisio",
 "ga":"Gaélico irlandés",
 "gd":"Gaélico",
 "gl":"Gallego",
 "gn":"Guaraní",
 "gu":"Gujarati",
 "gv":"Manés",
 "ha":"Hausa",
 "he":"Hebreo",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Croata",
 "hsb":"Alto sorabo",
 "hu":"Húngaro",
 "hy":"Armenio",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonesio",
 "ie":"Idioma occidental",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandés",
 "it":"Italiano",
 "iu":"Inuktitut",
 "ja":"Japonés",
 "jv":"Javanés",
 "ka":"Georgiano",
 "ki":"Kikuyu",
 "kk":"Kazajo",
 "kl":"Kalaallisut",
 "km":"Camboyano",
 "kn":"Kannada",
 "ko":"Coreano",
 "ks":"Kashmiri",
 "ku":"Kurdo",
 "kv":"Komi",
 "kw":"Cornuallés",
 "ky":"Kirguiz",
 "la":"Latín",
 "lb":"Luxemburgués",
 "li":"Limburgano",
 "ln":"Lingala",
 "lo":"Laosiano",
 "lt":"Lituano",
 "lv":"Letón",
 "mai":"Maithili",
 "mg":"Malagasio",
 "mh":"Marshalés",
 "mi":"Maorí",
 "mk":"Macedonio",
 "ml":"Malabar",
 "mn":"Mongol",
 "mo":"Moldavo",
 "mr":"Maratí",
 "ms":"Malayo",
 "mt":"Maltés",
 "my":"Burmés",
 "na":"Nauru",
 "nb":"Noruego Bokmål",
 "nd":"Ndebele del norte",
 "nds":"Bajo sajón",
 "ne":"Nepalí",
 "ng":"Ndonga",
 "nl":"Holandés",
 "nn":"Noruego Nynorsk",
 "nr":"Ndebele del sur",
 "nso":"Sotho del norte",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Occitano",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Osetio",
 "pa":"Punjabi",
 "pi":"Palí",
 "pl":"Polaco",
 "ps":"Pastún",
 "pt":"Portugués",
 "pt_BR":"Portugués brasileño",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Rumano",
 "rom":"Rumaní",
 "ru":"Ruso",
 "rw":"Kinyarwanda",
 "sa":"Sánscrito",
 "sc":"Sardo",
 "sd":"Sindhi",
 "se":"Samí del norte",
 "sg":"Sango",
 "si":"Cingalés",
 "sk":"Eslovaco",
 "sl":"Esloveno",
 "sm":"Samoano",
 "sn":"Shona",
 "so":"Somalí",
 "sq":"Albanés",
 "sr":"Serbio",
 "sr@ijekavian":"Serbio ijekaviano",
 "sr@ijekavianlatin":"Serbio latino ijekaviano",
 "sr@latin":"Serbio latino",
 "ss":"Swati",
 "st":"Sotho del sur",
 "su":"Sudanés",
 "sv":"Sueco",
 "sw":"Suahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tayiko",
 "th":"Tailandés",
 "ti":"Tigrinya",
 "tk":"Turkmenio",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turco",
 "ts":"Tsonga",
 "tt":"Tatar",
 "tw":"Twi",
 "ty":"Tahitiano",
 "ug":"Uigur",
 "uk":"Ucraniano",
 "ur":"Urdu",
 "uz":"Uzbeko",
 "uz@cyrillic":"Uzbeko (Cirílico)",
 "ven":"Venda",
 "vi":"Vietnamita",
 "vo":"Volapük",
 "wa":"Walloon",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Yiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chino",
 "zh_CN":"Chino simplificado",
 "zh_HK":"Chino (Hong Kong)",
 "zh_TW":"Chino tradicional",
 "zu":"Zulú",
},
"fr": {
 "aa":"Afar",
 "ab":"Abkhaze",
 "ae":"Avestan",
 "af":"Afrikaans",
 "am":"Amharique",
 "ar":"Arabe",
 "as":"Assamais",
 "ast":"Asturien",
 "ay":"Aymara",
 "az":"Azerbaïdjanais",
 "ba":"Bachkir",
 "be":"Biélorusse",
 "be@latin":"Biélorusse (Latin)",
 "bg":"Bulgare",
 "bh":"Bihari",
 "bi":"Bichlamar",
 "bn":"Bengali",
 "bn_IN":"Bengali (Inde)",
 "bo":"Tibétain",
 "br":"Breton",
 "bs":"Bosniaque",
 "ca":"Catalan",
 "ca@valencia":"Catalan (Valence)",
 "ce":"Tchétchène",
 "ch":"Chamorro",
 "co":"Corse",
 "crh":"Tatar de Crimée",
 "cs":"Tchèque",
 "csb":"Kachoube",
 "cu":"Vieux-slave liturgique",
 "cv":"Tchouvache",
 "cy":"Gallois",
 "da":"Danois",
 "de":"Allemand",
 "dsb":"Bas-Sorabe",
 "dz":"Dzongkha",
 "el":"Grec",
 "en":"Anglais",
 "en_GB":"Anglais britannique",
 "en_US":"Anglais américain",
 "eo":"Espéranto",
 "es":"Espagnol",
 "et":"Estonien",
 "eu":"Basque",
 "fa":"Persan",
 "fi":"Finnois",
 "fj":"Fidjien",
 "fo":"Féroïen",
 "fr":"Français",
 "fy":"Frison",
 "ga":"Gaélique irlandais",
 "gd":"Gaélique",
 "gl":"Galicien",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Mannois",
 "ha":"Haoussa",
 "he":"Hébreu",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri motu",
 "hr":"Croate",
 "hsb":"Haut-Sorabe",
 "hu":"Hongrois",
 "hy":"Arménien",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonésien",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandais",
 "it":"Italien",
 "iu":"Inuktitut",
 "ja":"Japonais",
 "jv":"Javanais",
 "ka":"Géorgien",
 "ki":"Kikuyu",
 "kk":"Kazakh",
 "kl":"Kalaallisut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Coréen",
 "ks":"Kashmiri",
 "ku":"Kurde",
 "kv":"Komi",
 "kw":"Cornique",
 "ky":"Kirghize",
 "la":"Latin",
 "lb":"Luxembourgeois",
 "li":"Limbourgeois",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Lituanien",
 "lv":"Letton",
 "mai":"Maïthili",
 "mg":"Malgache",
 "mh":"Marshall",
 "mi":"Maori",
 "mk":"Macédonien",
 "ml":"Malayalam",
 "mn":"Mongol",
 "mo":"Moldave",
 "mr":"Marathe",
 "ms":"Malais",
 "mt":"Maltais",
 "my":"Birman",
 "na":"Nauruan",
 "nb":"Norvégien Bokmaal",
 "nd":"Ndébélé du Nord",
 "nds":"Bas saxon",
 "ne":"Népalais",
 "ng":"Ndonga",
 "nl":"Hollandais",
 "nn":"Norvégien Nynorsk",
 "nr":"Ndébélé du Sud",
 "nso":"Sotho du Nord",
 "nv":"Navaho",
 "ny":"Chichewa",
 "oc":"Occitan",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Ossète",
 "pa":"Panjabi / Punjabi",
 "pi":"Pali",
 "pl":"Polonais",
 "ps":"Pushto",
 "pt":"Portugais",
 "pt_BR":"Portugais Brésilien",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Roumain",
 "rom":"Rromani",
 "ru":"Russe",
 "rw":"Kinyarwanda",
 "sa":"Sanskrit",
 "sc":"Sarde",
 "sd":"Sindhi",
 "se":"Sami du Nord",
 "sg":"Sango",
 "si":"Cingalais",
 "sk":"Slovaque",
 "sl":"Slovène",
 "sm":"Samoan",
 "sn":"Shona",
 "so":"Somali",
 "sq":"Albanais",
 "sr":"Serbe",
 "sr@ijekavian":"Serbe Ijékavien",
 "sr@ijekavianlatin":"Serbe Ijékavien Latin",
 "sr@latin":"Serbe Latin",
 "ss":"Swati",
 "st":"Sotho du Sud",
 "su":"Soudanais",
 "sv":"Suédois",
 "sw":"Swahili",
 "ta":"Tamoul",
 "te":"Télougou",
 "tg":"Tadjik",
 "th":"Thaïlandais",
 "ti":"Tigrigna",
 "tk":"Turkmène",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turc",
 "ts":"Tsonga",
 "tt":"Tatar",
 "tw":"Twi",
 "ty":"Tahitien",
 "ug":"Ouïghour",
 "uk":"Ukrainien",
 "ur":"Urdu",
 "uz":"Ouzbek",
 "uz@cyrillic":"Ouzbek (Cyrillique)",
 "ven":"Venda",
 "vi":"Vietnamien",
 "vo":"Volapük",
 "wa":"Wallon",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Yiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chinois",
 "zh_CN":"Chinois simplifié",
 "zh_HK":"Chinois (Hong Kong)",
 "zh_TW":"Chinois traditionnel",
 "zu":"Zoulou",
},
"gl": {
 "aa":"Afar",
 "ab":"Abkhazo",
 "ae":"Avéstico",
 "af":"Africáner",
 "am":"Amhárico",
 "ar":"Árabe",
 "as":"Asamés",
 "ast":"Asturiano",
 "ay":"Aimará",
 "az":"Acerbaixano",
 "ba":"Baxequir",
 "be":"Bielorruso",
 "be@latin":"Bielorruso (latino)",
 "bg":"Búlgaro",
 "bh":"Biari",
 "bi":"Bislamá",
 "bn":"Bengalí",
 "bn_IN":"Bengalí (India)",
 "bo":"Tibetano",
 "br":"Bretón",
 "bs":"Bosníaco",
 "ca":"Catalán",
 "ca@valencia":"Catalán (valenciano)",
 "ce":"Tchecheno",
 "ch":"Chamorro",
 "co":"Corso",
 "crh":"Tártaro da Crimea",
 "cs":"Checo",
 "csb":"Caxubo",
 "cu":"Eslavo eclesiástico",
 "cv":"Chuvash",
 "cy":"Galés",
 "da":"Dinamarqués",
 "de":"Alemán",
 "dsb":"Baixo sorabio",
 "dz":"Dzongkha",
 "el":"Grego",
 "en":"Inglés",
 "en_GB":"Inglés británico",
 "en_US":"Inglés americano",
 "eo":"Esperanto",
 "es":"Castelán",
 "et":"Estoniano",
 "eu":"Vasco",
 "fa":"Frisón (Parsi)",
 "fi":"Finlandés",
 "fj":"Fixiano",
 "fo":"Feroés",
 "fr":"Francés",
 "fy":"Frisio",
 "ga":"Gaélico irlandés",
 "gd":"Gaélico",
 "gl":"Galego",
 "gn":"Guaraní",
 "gu":"Guzerate",
 "gv":"Manés",
 "ha":"Hausa",
 "he":"Hebreo",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Croata",
 "hsb":"Alto sorabio",
 "hu":"Húngaro",
 "hy":"Armenio",
 "hz":"Hereró",
 "ia":"Interlingua",
 "id":"Indonesio",
 "ie":"Interlingua",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandés",
 "it":"Italiano",
 "iu":"Inuktitut",
 "ja":"Xaponés",
 "jv":"Xavanés",
 "ka":"Xeorxiano",
 "ki":"Gikuyu",
 "kk":"Cazaxo",
 "kl":"Groenlandés",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Coreano",
 "ks":"Caxemir",
 "ku":"Kurdo",
 "kv":"Komi",
 "kw":"Córnico",
 "ky":"Quirguiz",
 "la":"Latín",
 "lb":"Luxemburgués",
 "li":"Limburgués",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Lituano",
 "lv":"Letón",
 "mai":"Maithili",
 "mg":"Malgaxe",
 "mh":"Marshalés",
 "mi":"Maorí",
 "mk":"Macedonio",
 "ml":"Malaiala",
 "mn":"Mongol",
 "mo":"Moldavo",
 "mr":"Marata",
 "ms":"Malaio",
 "mt":"Maltés",
 "my":"Burmés",
 "na":"Nauru",
 "nb":"Noruegués Bokmål",
 "nd":"Ndebele do norte",
 "nds":"Baixo saxón",
 "ne":"Nepalí",
 "ng":"Ndonga",
 "nl":"Holandés",
 "nn":"Noruegués Nynorsk",
 "nr":"Ndebele do sur",
 "nso":"Sisotho do norte",
 "nv":"Navajo",
 "ny":"Chewa",
 "oc":"Occitano",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Oseto",
 "pa":"Panxabi",
 "pi":"Pali",
 "pl":"Polaco",
 "ps":"Pashto",
 "pt":"Portugués",
 "pt_BR":"Portugués do Brasil",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Romanés",
 "rom":"Romaní",
 "ru":"Ruso",
 "rw":"Quiñaruanda",
 "sa":"Sánscrito",
 "sc":"Sardo",
 "sd":"Sindhi",
 "se":"Sami do norte",
 "sg":"Sango",
 "si":"Cingalés",
 "sk":"Eslovaco",
 "sl":"Esloveno",
 "sm":"Samoano",
 "sn":"Chona",
 "so":"Somalí",
 "sq":"Albanés",
 "sr":"Serbio",
 "sr@ijekavian":"Serbio ixtocavio",
 "sr@ijekavianlatin":"Serbio ishtocavio Latino",
 "sr@latin":"Serbio latino",
 "ss":"Swati",
 "st":"Xisotho do sur",
 "su":"Sudanés",
 "sv":"Sueco",
 "sw":"Suahili",
 "ta":"Tamil",
 "te":"Telugúo",
 "tg":"Taxico",
 "th":"Tailandés",
 "ti":"Tigriñán",
 "tk":"Turcomeno",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turco",
 "ts":"Xitsonga",
 "tt":"Tártaro",
 "tw":"Twi",
 "ty":"Tahitiano",
 "ug":"Uigure",
 "uk":"Ucraíno",
 "ur":"Urdú",
 "uz":"Usbeco",
 "uz@cyrillic":"Usbeco (cirílico)",
 "ven":"Venda",
 "vi":"Vietnamita",
 "vo":"Volapük",
 "wa":"Valón",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Iídiche",
 "yo":"Iorubá",
 "za":"Zhuang",
 "zh":"Chinés",
 "zh_CN":"Chinés simplificado",
 "zh_HK":"Chinés (Hong Kong)",
 "zh_TW":"Chinés tradicional",
 "zu":"Zulú",
},
"hu": {
 "aa":"Afar",
 "ab":"Abház",
 "ae":"Avesztáni",
 "af":"Afrikaans",
 "am":"Amhár",
 "ar":"Arab",
 "as":"Asszámi",
 "ast":"Asztúriai",
 "ay":"Ajmara",
 "az":"Azerbajdzsán",
 "ba":"Baskír",
 "be":"Belorusz",
 "be@latin":"Belorusz (latin betűs)",
 "bg":"Bolgár",
 "bh":"Bihari",
 "bi":"Biszlama",
 "bn":"Bengáli",
 "bn_IN":"Bengáli (India)",
 "bo":"Tibeti",
 "br":"Breton",
 "bs":"Bosnyák",
 "ca":"Katalán",
 "ca@valencia":"Katalán (valenciai)",
 "ce":"Csecsen",
 "ch":"Chamorro",
 "co":"Korzikai",
 "crh":"Krími tatár",
 "cs":"Cseh",
 "csb":"Kasub",
 "cu":"Szláv (egyházi)",
 "cv":"Csuvas",
 "cy":"Velszi",
 "da":"Dán",
 "de":"Német",
 "dsb":"Alsó szorb",
 "dz":"Dzongkha",
 "el":"Görög",
 "en":"Angol",
 "en_GB":"Angol (brit)",
 "en_US":"Angol (amerikai)",
 "eo":"Eszperantó",
 "es":"Spanyol",
 "et":"Észt",
 "eu":"Baszk",
 "fa":"Fárszi (perzsa)",
 "fi":"Finn",
 "fj":"Fidzsi",
 "fo":"Faröei",
 "fr":"Francia",
 "fy":"Fríz",
 "ga":"Gall (ír)",
 "gd":"Gall",
 "gl":"Galíciai",
 "gn":"Guarani",
 "gu":"Gudzsarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Héber",
 "hi":"Hindi",
 "hne":"Cshattíszgarhi",
 "ho":"Hiri motu",
 "hr":"Horvát",
 "hsb":"Felső szorb",
 "hu":"Magyar",
 "hy":"Örmény",
 "hz":"Hereró",
 "ia":"Interlingua",
 "id":"Indonéz",
 "ie":"Interlingue",
 "ik":"Inupiak",
 "io":"Ido",
 "is":"Izlandi",
 "it":"Olasz",
 "iu":"Inuktitut",
 "ja":"Japán",
 "jv":"Jávai",
 "ka":"Grúz",
 "ki":"Kikuju",
 "kk":"Kazah",
 "kl":"Kalaalliszut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Koreai",
 "ks":"Kasmír",
 "ku":"Kurd",
 "kv":"Komi",
 "kw":"Korn",
 "ky":"Kirgiz",
 "la":"Latin",
 "lb":"Luxemburgi",
 "li":"Limburgi",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Litván",
 "lv":"Lett",
 "mai":"Maithili",
 "mg":"Malagazi",
 "mh":"Marsalli",
 "mi":"Maori",
 "mk":"Macedón",
 "ml":"Malajalam",
 "mn":"Mongol",
 "mo":"Moldáv",
 "mr":"Marati",
 "ms":"Maláj",
 "mt":"Máltai",
 "my":"Burmai",
 "na":"Nauru",
 "nb":"Norvég (bokmal)",
 "nd":"Ndebele (északi)",
 "nds":"Alsószász",
 "ne":"Nepáli",
 "ng":"Ndonga",
 "nl":"Holland",
 "nn":"Norvég (nynorsk)",
 "nr":"Ndebele (déli)",
 "nso":"Északi sotho",
 "nv":"Navajo",
 "ny":"Csicseva",
 "oc":"Okcitán",
 "om":"Oromo",
 "or":"Orija",
 "os":"Oszét",
 "pa":"Pandzsabi",
 "pi":"Pali",
 "pl":"Lengyel",
 "ps":"Pusto",
 "pt":"Portugál",
 "pt_BR":"Portugál (brazil)",
 "qu":"Kecsua",
 "rn":"Rundi",
 "ro":"Román",
 "rom":"Lovári cigány",
 "ru":"Orosz",
 "rw":"Kinjarvanda",
 "sa":"Szanszkrit",
 "sc":"Szardíniai",
 "sd":"Szindi",
 "se":"Északi szami",
 "sg":"Szangó",
 "si":"Szingaléz",
 "sk":"Szlovák",
 "sl":"Szlovén",
 "sm":"Szamoai",
 "sn":"Sona",
 "so":"Szomáli",
 "sq":"Albán",
 "sr":"Szerb",
 "sr@ijekavian":"Szerb (ijekavica)",
 "sr@ijekavianlatin":"Szerb (ijekavica, latin betűs)",
 "sr@latin":"Szerb (latin betűs)",
 "ss":"Szvati",
 "st":"Sotho (déli)",
 "su":"Szundanéz",
 "sv":"Svéd",
 "sw":"Szvahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tádzsik",
 "th":"Thai",
 "ti":"Tigrinja",
 "tk":"Türkmén",
 "tn":"Tszvana",
 "to":"Tonga",
 "tr":"Török",
 "ts":"Tsonga",
 "tt":"Tatár",
 "tw":"Tvi",
 "ty":"Tahiti",
 "ug":"Ujgur",
 "uk":"Ukrán",
 "ur":"Urdu",
 "uz":"Üzbég",
 "uz@cyrillic":"Üzbég (cirill)",
 "ven":"Venda",
 "vi":"Vietnami",
 "vo":"Volapük",
 "wa":"Vallon",
 "wo":"Volof",
 "xh":"Xhosza",
 "yi":"Jiddis",
 "yo":"Joruba",
 "za":"Zhuang",
 "zh":"Kínai",
 "zh_CN":"Kínai (egyszerűsített)",
 "zh_HK":"Kínai (hongkongi)",
 "zh_TW":"Kínai (hagyományos)",
 "zu":"Zulu",
},
"it": {
 "aa":"Afar",
 "ab":"Abcaso",
 "ae":"Avestano",
 "af":"Afrikaans",
 "am":"Amarico",
 "ar":"Arabo",
 "as":"Assamese",
 "ast":"Asturiano",
 "ay":"Aymara",
 "az":"Azero",
 "ba":"Baškiro",
 "be":"Bielorusso",
 "be@latin":"Bielorusso (Latino)",
 "bg":"Bulgaro",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengalese",
 "bn_IN":"Bengalese (India)",
 "bo":"Tibetano",
 "br":"Bretone",
 "bs":"Bosniaco",
 "ca":"Catalano",
 "ca@valencia":"Catalano (Valenziano)",
 "ce":"Ceceno",
 "ch":"Chamorro",
 "co":"Corso",
 "crh":"Tataro di Crimea",
 "cs":"Ceco",
 "csb":"Casciubico",
 "cu":"Slavo ecclesiastico",
 "cv":"Ciuvascio",
 "cy":"Gallese",
 "da":"Danese",
 "de":"Tedesco",
 "dsb":"Basso sorabo",
 "dz":"Dzongkha",
 "el":"Greco",
 "en":"Inglese",
 "en_GB":"Inglese britannico",
 "en_US":"Inglese americano",
 "eo":"Esperanto",
 "es":"Spagnolo",
 "et":"Estone",
 "eu":"Basco",
 "fa":"Farsi (Persiano)",
 "fi":"Finlandese",
 "fj":"Figiano",
 "fo":"Faroese",
 "fr":"Francese",
 "fy":"Frisone",
 "ga":"Gaelico irlandese",
 "gd":"Gaelico",
 "gl":"Gallego",
 "gn":"Guaraní",
 "gu":"Gujarati",
 "gv":"Mannese",
 "ha":"Hausa",
 "he":"Ebraico",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri motu",
 "hr":"Croato",
 "hsb":"Alto sorabo",
 "hu":"Ungherese",
 "hy":"Armeno",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonesiano",
 "ie":"Interlingue",
 "ik":"Inupiak",
 "io":"Ido",
 "is":"Islandese",
 "it":"Italiano",
 "iu":"Inuktitut",
 "ja":"Giapponese",
 "jv":"Giavanese",
 "ka":"Georgiano",
 "ki":"Kikuyu",
 "kk":"Kazako",
 "kl":"Groenlandese",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Coreano",
 "ks":"Kašmiro",
 "ku":"Curdo",
 "kv":"Komi",
 "kw":"Cornico",
 "ky":"Kirghiso",
 "la":"Latino",
 "lb":"Lussemburghese",
 "li":"Limburghese",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Lituano",
 "lv":"Lettone",
 "mai":"Maithili",
 "mg":"Malgascio",
 "mh":"Marsciallese",
 "mi":"Maori",
 "mk":"Macedone",
 "ml":"Malayalam",
 "mn":"Mongolo",
 "mo":"Moldavo",
 "mr":"Marathi",
 "ms":"Malese",
 "mt":"Maltese",
 "my":"Birmano",
 "na":"Nauruano",
 "nb":"Norvegese bokmål",
 "nd":"Ndebele settentrionale",
 "nds":"Basso sassone",
 "ne":"Nepalese",
 "ng":"Ndonga",
 "nl":"Olandese",
 "nn":"Norvegese nynorsk",
 "nr":"Ndebele meridionale",
 "nso":"Sesotho del nord",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Occitano",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Osseto",
 "pa":"Panjabi",
 "pi":"Pali",
 "pl":"Polacco",
 "ps":"Paštu",
 "pt":"Portoghese",
 "pt_BR":"Portoghese brasiliano",
 "qu":"Quechua",
 "rn":"Kirundi",
 "ro":"Rumeno",
 "rom":"Romaní",
 "ru":"Russo",
 "rw":"Kinyarwanda",
 "sa":"Sanscrito",
 "sc":"Sardo",
 "sd":"Sindhi",
 "se":"Sami settentrionale",
 "sg":"Sango",
 "si":"Singalese",
 "sk":"Slovacco",
 "sl":"Sloveno",
 "sm":"Samoano",
 "sn":"Shona",
 "so":"Somalo",
 "sq":"Albanese",
 "sr":"Serbo",
 "sr@ijekavian":"Serbo ijekavo",
 "sr@ijekavianlatin":"Serbo ijekavo latino",
 "sr@latin":"Serbo latino",
 "ss":"Swati",
 "st":"Sesotho meridionale",
 "su":"Sondanese",
 "sv":"Svedese",
 "sw":"Swahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tagico",
 "th":"Thailandese",
 "ti":"Tigrino",
 "tk":"Turkmeno",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turco",
 "ts":"Tsonga",
 "tt":"Tataro",
 "tw":"Twi",
 "ty":"Tahitiano",
 "ug":"Uiguro",
 "uk":"Ucraino",
 "ur":"Urdu",
 "uz":"Usbeco",
 "uz@cyrillic":"Usbeco (cirillico)",
 "ven":"Venda",
 "vi":"Vietnamita",
 "vo":"Volapük",
 "wa":"Vallone",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Yiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Cinese",
 "zh_CN":"Cinese semplificato",
 "zh_HK":"Cinese (Hong Kong)",
 "zh_TW":"Cinese tradizionale",
 "zu":"Zulu",
},
"ja": {
 "aa":"アファル語",
 "ab":"アブハズ語",
 "ae":"アベスタ語",
 "af":"アフリカーンス語",
 "am":"アムハラ語",
 "ar":"アラビア語",
 "as":"アッサム語",
 "ast":"アストゥリアス語",
 "ay":"アイマラ語",
 "az":"アゼルバイジャン語",
 "ba":"バシュキール語",
 "be":"ベラルーシ語",
 "be@latin":"ベラルーシ語 (ラテン文字)",
 "bg":"ブルガリア語",
 "bh":"ビハール語",
 "bi":"ビスラマ語",
 "bn":"ベンガル語",
 "bn_IN":"ベンガル語 (インド)",
 "bo":"チベット語",
 "br":"ブルトン語",
 "bs":"ボスニア語",
 "ca":"カタロニア語",
 "ca@valencia":"バレンシア語",
 "ce":"チェチェン語",
 "ch":"チャモロ語",
 "co":"コルシカ語",
 "crh":"クリミア・タタール語",
 "cs":"チェコ語",
 "csb":"カシューブ語",
 "cu":"教会スラブ語",
 "cv":"チュワシュ語",
 "cy":"ウェールズ語",
 "da":"デンマーク語",
 "de":"ドイツ語",
 "dsb":"下ソルブ語",
 "dz":"ゾンカ語",
 "el":"ギリシャ語",
 "en":"英語",
 "en_GB":"イギリス英語",
 "en_US":"アメリカ英語",
 "eo":"エスペラント語",
 "es":"スペイン語",
 "et":"エストニア語",
 "eu":"バスク語",
 "fa":"ペルシア語",
 "fi":"フィンランド語",
 "fj":"フィジー語",
 "fo":"フェーロー語",
 "fr":"フランス語",
 "fy":"フリジア語",
 "ga":"アイルランド系ゲール語",
 "gd":"ゲール語",
 "gl":"ガリシア語",
 "gn":"グァラニ語",
 "gu":"グジャラート語",
 "gv":"マン島語",
 "ha":"ハウサ語",
 "he":"ヘブライ語",
 "hi":"ヒンディー語",
 "hne":"チャッティースガリー語",
 "ho":"ヒリモトゥ語",
 "hr":"クロアチア語",
 "hsb":"上ソルブ語",
 "hu":"ハンガリー語",
 "hy":"アルメニア語",
 "hz":"ヘレロ語",
 "ia":"インターリンガ",
 "id":"インドネシア語",
 "ie":"インターリング",
 "ik":"イヌピアック語",
 "io":"イド語",
 "is":"アイスランド語",
 "it":"イタリア語",
 "iu":"イヌイット語",
 "ja":"日本語",
 "jv":"ジャワ語",
 "ka":"グルジア語",
 "ki":"キクユ語",
 "kk":"カザフ語",
 "kl":"グリーンランド語",
 "km":"クメール語",
 "kn":"カンナダ語",
 "ko":"韓国語・朝鮮語",
 "ks":"カシュミール語",
 "ku":"クルド語",
 "kv":"コミ語",
 "kw":"コーンウォール語",
 "ky":"キルギス語",
 "la":"ラテン語",
 "lb":"ルクセンブルク語",
 "li":"リグリア語",
 "ln":"リンガラ語",
 "lo":"ラオ語",
 "lt":"リトアニア語",
 "lv":"ラトビア語",
 "mai":"マイティリー語",
 "mg":"マラガシ語",
 "mh":"マーシャル語",
 "mi":"マオリ語",
 "mk":"マケドニア語",
 "ml":"マラヤーラム語",
 "mn":"モンゴル語",
 "mo":"モルダビア語",
 "mr":"マラーティー語",
 "ms":"マレー語",
 "mt":"マルタ語",
 "my":"ビルマ語",
 "na":"ナウル語",
 "nb":"ノルウェー語 (ブークモール)",
 "nd":"北ンデベレ語",
 "nds":"低ザクセン語",
 "ne":"ネパール語",
 "ng":"ンドゥンガ語",
 "nl":"オランダ語",
 "nn":"ノルウェー語 (ニーノシュク)",
 "nr":"南ンデベレ語",
 "nso":"北ソト語",
 "nv":"ナバホ語",
 "ny":"チェワ語",
 "oc":"オック語",
 "om":"オロモ語",
 "or":"オリヤー語",
 "os":"オセット語",
 "pa":"パンジャーブ語",
 "pi":"パーリ語",
 "pl":"ポーランド語",
 "ps":"パシュトー語",
 "pt":"ポルトガル語",
 "pt_BR":"ブラジル ポルトガル語",
 "qu":"ケチュア語",
 "rn":"ルンディ語",
 "ro":"ルーマニア語",
 "rom":"ロマ語",
 "ru":"ロシア語",
 "rw":"ルワンダ語",
 "sa":"サンスクリット語",
 "sc":"サルデーニャ語",
 "sd":"シンド語",
 "se":"北サーミ語",
 "sg":"サンゴ語",
 "si":"シンハラ語",
 "sk":"スロバキア語",
 "sl":"スロベニア語",
 "sm":"サモア語",
 "sn":"ショナ語",
 "so":"ソマリ語",
 "sq":"アルバニア語",
 "sr":"セルビア語",
 "sr@ijekavian":"セルビア語イェ方言",
 "sr@ijekavianlatin":"セルビア語イェ方言 (ラテン文字)",
 "sr@latin":"セルビア語 (ラテン文字)",
 "ss":"スワティ語",
 "st":"南ソト語",
 "su":"スンダ語",
 "sv":"スウェーデン語",
 "sw":"スワヒリ語",
 "ta":"タミル語",
 "te":"テルグ語",
 "tg":"タジク語",
 "th":"タイ語",
 "ti":"ティグリニア語",
 "tk":"トルクメン語",
 "tn":"ツワナ語",
 "to":"トンガ語",
 "tr":"トルコ語",
 "ts":"ツォンガ語",
 "tt":"タタール語",
 "tw":"チュイ語",
 "ty":"タヒチ語",
 "ug":"ウイグル語",
 "uk":"ウクライナ語",
 "ur":"ウルドゥー語",
 "uz":"ウズベク語",
 "uz@cyrillic":"ウズベク語 (キリル文字)",
 "ven":"ベンダ語",
 "vi":"ベトナム語",
 "vo":"ボラピューク語",
 "wa":"ワロン語",
 "wo":"ウォロフ語",
 "xh":"コサ語",
 "yi":"イディッシュ語",
 "yo":"ヨルバ語",
 "za":"チュワン語",
 "zh":"中国語",
 "zh_CN":"中国語 簡体字",
 "zh_HK":"中国語 (香港)",
 "zh_TW":"中国語 繁体字",
 "zu":"ズールー語",
},
"nl": {
 "aa":"Afar",
 "ab":"Abchazisch",
 "ae":"Avestisch",
 "af":"Afrikaans",
 "am":"Amharisch",
 "ar":"Arabisch",
 "as":"Assamees",
 "ast":"Asturisch",
 "ay":"Aymara",
 "az":"Azerbeidzjaans",
 "ba":"Basjkiers",
 "be":"Wit-Russisch",
 "be@latin":"Wit-Russisch (Latijn)",
 "bg":"Bulgaars",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengaals",
 "bn_IN":"Bengaals (India)",
 "bo":"Tibetaans",
 "br":"Bretons",
 "bs":"Bosnisch",
 "ca":"Catalaans",
 "ca@valencia":"Catalaans (Valentiaans)",
 "ce":"Tsjetsjeens",
 "ch":"Chamorro",
 "co":"Corsicaans",
 "crh":"Krim-Tataars",
 "cs":"Tsjechisch",
 "csb":"Kasjoebisch",
 "cu":"Kerk Slavisch",
 "cv":"Tsjoevasjisch",
 "cy":"Welsh",
 "da":"Deens",
 "de":"Duits",
 "dsb":"Nedersorbisch",
 "dz":"Dzongkha",
 "el":"Grieks",
 "en":"Engels",
 "en_GB":"Brits Engels",
 "en_US":"Engels (Amerikaans)",
 "eo":"Esperanto",
 "es":"Spaans",
 "et":"Ests",
 "eu":"Baskisch",
 "fa":"Farsi (Perzisch)",
 "fi":"Fins",
 "fj":"Fijisch",
 "fo":"Faeröers",
 "fr":"Frans",
 "fy":"Fries",
 "ga":"Gaelic (Iers)",
 "gd":"Gaelic",
 "gl":"Galicisch",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Hebreeuws",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Kroatisch",
 "hsb":"Oppersorbisch",
 "hu":"Hongaars",
 "hy":"Armeens",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonesisch",
 "ie":"Interlingue",
 "ik":"Inupiak",
 "io":"Ido",
 "is":"IJslands",
 "it":"Italiaans",
 "iu":"Inuktitut",
 "ja":"Japans",
 "jv":"Javaans",
 "ka":"Georgisch",
 "ki":"Kikuyu",
 "kk":"Kazachs",
 "kl":"Kalaallisut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Koreaans",
 "ks":"Kashmiri",
 "ku":"Koerdisch",
 "kv":"Komi",
 "kw":"Cornish",
 "ky":"Kirgizisch",
 "la":"Latijn",
 "lb":"Luxemburgs",
 "li":"Limburgs",
 "ln":"Lingala",
 "lo":"Laotiaans",
 "lt":"Litouws",
 "lv":"Lets",
 "mai":"Maithili",
 "mg":"Madagaskar",
 "mh":"Marshallees",
 "mi":"Maori",
 "mk":"Macedonisch",
 "ml":"Malayalam",
 "mn":"Mongools",
 "mo":"Moldavisch",
 "mr":"Marathi",
 "ms":"Maleis",
 "mt":"Maltees",
 "my":"Burmees",
 "na":"Nauru",
 "nb":"Noors Bokmål",
 "nd":"Ndebele, Noord",
 "nds":"Nedersaksisch",
 "ne":"Nepalees",
 "ng":"Ndonga",
 "nl":"Nederlands",
 "nn":"Noors Nynorsk",
 "nr":"Ndebele, Zuid",
 "nso":"Noord-Sotho",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Occitan",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Ossetisch",
 "pa":"Panjabi",
 "pi":"Pali",
 "pl":"Pools",
 "ps":"Poesjto",
 "pt":"Portugees",
 "pt_BR":"Braziliaans Portugees",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Roemeens",
 "rom":"Roma",
 "ru":"Russisch",
 "rw":"Kinyarwanda",
 "sa":"Sanskriet",
 "sc":"Sardinisch",
 "sd":"Sindhi",
 "se":"Noord-Sami",
 "sg":"Sango",
 "si":"Singalees",
 "sk":"Slowaaks",
 "sl":"Sloveens",
 "sm":"Samoaans",
 "sn":"Shona",
 "so":"Somalisch",
 "sq":"Albanees",
 "sr":"Servisch",
 "sr@ijekavian":"Servisch Ijekavian",
 "sr@ijekavianlatin":"Servisch Ijekavian latijn",
 "sr@latin":"Servisch latijn",
 "ss":"Swati",
 "st":"Zuid-Sothaans",
 "su":"Soendanees",
 "sv":"Zweeds",
 "sw":"Swahili",
 "ta":"Tamil",
 "te":"Telugu",
 "tg":"Tadzjieks",
 "th":"Thai",
 "ti":"Tigrinya",
 "tk":"Turkmeens",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turks",
 "ts":"Tsonga",
 "tt":"Tataars",
 "tw":"Twi",
 "ty":"Tahitisch",
 "ug":"Oeigoers",
 "uk":"Oekraïens",
 "ur":"Urdu",
 "uz":"Oezbeeks",
 "uz@cyrillic":"Oezbeeks (Cyrilisch)",
 "ven":"Venda",
 "vi":"Vietnamees",
 "vo":"Volapúk",
 "wa":"Waals",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Jiddisch",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chinees",
 "zh_CN":"Vereenvoudigd Chinees",
 "zh_HK":"Chinees (Hong Kong)",
 "zh_TW":"Traditioneel Chinees",
 "zu":"Zulu",
},
"pl": {
 "aa":"Afarski",
 "ab":"Abchaski",
 "ae":"Awestański (Irański)",
 "af":"Afrykanerski",
 "am":"Etiopski",
 "ar":"Arabski",
 "as":"Assamijski",
 "ast":"Asturyjski",
 "ay":"Aymara",
 "az":"Azerbejdżański",
 "ba":"Baszkirski",
 "be":"Białoruski",
 "be@latin":"Białoruski (alfabet łaciński)",
 "bg":"Bułgarski",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengalski",
 "bn_IN":"Bengalski (Indie)",
 "bo":"Tybetański",
 "br":"Bretoński",
 "bs":"Bośniacki",
 "ca":"Kataloński",
 "ca@valencia":"Kataloński (walencki)",
 "ce":"Czeczeński",
 "ch":"Chamorro",
 "co":"Korsykański",
 "crh":"Tatarski (Krymski)",
 "cs":"Czeski",
 "csb":"Kaszubski",
 "cu":"Starocerkiewno-słowiański",
 "cv":"Czuwaski",
 "cy":"Walijski",
 "da":"Duński",
 "de":"Niemiecki",
 "dsb":"Dolnołużycki",
 "dz":"Dzongkha",
 "el":"Grecki",
 "en":"Angielski",
 "en_GB":"Angielski brytyjski",
 "en_US":"Angielski amerykański",
 "eo":"Esperanto",
 "es":"Hiszpański",
 "et":"Estoński",
 "eu":"Baskijski",
 "fa":"Farsi (Perski)",
 "fi":"Fiński",
 "fj":"z Fiji",
 "fo":"z Wysp Owczych",
 "fr":"Francuski",
 "fy":"Fryzyjski",
 "ga":"Irlandzki Gaelic",
 "gd":"celtycki (gaelic)",
 "gl":"Galicyjski",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"Hebrajski",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Chorwacki",
 "hsb":"Górnołużycki",
 "hu":"Węgierski",
 "hy":"Ormiański",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"Indonezyjski",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandzki",
 "it":"Włoski",
 "iu":"Inuktitut",
 "ja":"Japoński",
 "jv":"Jawański",
 "ka":"Gruziński",
 "ki":"Kikuyu",
 "kk":"Kazachski",
 "kl":"Kalaallisut",
 "km":"Khmerski",
 "kn":"Kannada",
 "ko":"Koreański",
 "ks":"Kaszmirski",
 "ku":"Kurdyjski",
 "kv":"z Komi",
 "kw":"Kornwalijski",
 "ky":"Kirgiski",
 "la":"Łaciński",
 "lb":"Luksemburski",
 "li":"Limburgiański",
 "ln":"Lingala",
 "lo":"Laotański",
 "lt":"Litewski",
 "lv":"Łotewski",
 "mai":"Maithili",
 "mg":"Madagaskarski",
 "mh":"z Wysp Marshalla",
 "mi":"Maoryjski",
 "mk":"Macedoński",
 "ml":"Malajalam",
 "mn":"Mongolski",
 "mo":"Mołdawski",
 "mr":"Marathi (Hinduski)",
 "ms":"Malajski",
 "mt":"Maltański",
 "my":"Birmański",
 "na":"Nauru",
 "nb":"Norweski Bokmaal",
 "nd":"Ndebele, Północ",
 "nds":"Dolnosaksoński",
 "ne":"Nepalski",
 "ng":"Ndonga",
 "nl":"Holenderski",
 "nn":"Norweski Nynorsk",
 "nr":"Ndebele, Południe",
 "nso":"Północne Sotho",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Okcytański",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Osetański",
 "pa":"Pendżabski",
 "pi":"Pali",
 "pl":"Polski",
 "ps":"Pushto",
 "pt":"Portugalski",
 "pt_BR":"Portugalski (brazylijski)",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Rumuński",
 "rom":"Romski",
 "ru":"Rosyjski",
 "rw":"Kinyarwanda",
 "sa":"Sanskryt",
 "sc":"Sardyński",
 "sd":"Sindhi",
 "se":"Północny Sami",
 "sg":"Sango",
 "si":"Sinhala",
 "sk":"Słowacki",
 "sl":"Słoweński",
 "sm":"Samoański",
 "sn":"Shona",
 "so":"Somalijski",
 "sq":"Albański",
 "sr":"Serbski",
 "sr@ijekavian":"Serbski ijekawski",
 "sr@ijekavianlatin":"Serbski ijekawski łaciński",
 "sr@latin":"Serbski łaciński",
 "ss":"Swati",
 "st":"Sotho, Południowy",
 "su":"Sudański",
 "sv":"Szwedzki",
 "sw":"Swahili",
 "ta":"Tamiljski",
 "te":"Telugu",
 "tg":"Tadżycki",
 "th":"Tajski",
 "ti":"Tigrinya",
 "tk":"Turkmeński",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Turecki",
 "ts":"Tsonga",
 "tt":"Tatarski",
 "tw":"Twi",
 "ty":"Tahitański",
 "ug":"Ujgurski",
 "uk":"Ukraiński",
 "ur":"Urdu",
 "uz":"Uzbecki",
 "uz@cyrillic":"Uzbecki (cyrlica)",
 "ven":"Venda",
 "vi":"Wietnamski",
 "vo":"Volapük",
 "wa":"Waloński",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Jidysz",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Chiński",
 "zh_CN":"Chiński uproszczony",
 "zh_HK":"Chiński (Hong Kong)",
 "zh_TW":"Chiński tradycyjny",
 "zu":"Zuluski",
},
"pt_BR": {
 "aa":"Afar",
 "ab":"Abcázio",
 "ae":"Avéstico",
 "af":"Africâner",
 "am":"Amárico",
 "ar":"Árabe",
 "as":"Assamês",
 "ast":"Asturiano",
 "ay":"Aimará",
 "az":"Azeri",
 "ba":"Basquir",
 "be":"Bielorrusso",
 "be@latin":"Bielorrusso (latino)",
 "bg":"Búlgaro",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengali",
 "bn_IN":"Bengali (Índia)",
 "bo":"Tibetano",
 "br":"Bretão",
 "bs":"Bósnio",
 "ca":"Catalão",
 "ca@valencia":"Catalão (valenciano)",
 "ce":"Checheno",
 "ch":"Chamorro",
 "co":"Corso",
 "crh":"Tártaro da Crimeia",
 "cs":"Tcheco",
 "csb":"Cassúbia",
 "cu":"Eslavo eclesiástico",
 "cv":"Tchuvache",
 "cy":"Galês",
 "da":"Dinamarquês",
 "de":"Alemão",
 "dsb":"Baixo Sorábio",
 "dz":"Dzongkha",
 "el":"Grego",
 "en":"Inglês",
 "en_GB":"Inglês britânico",
 "en_US":"Inglês americano",
 "eo":"Esperanto",
 "es":"Espanhol",
 "et":"Estoniano",
 "eu":"Basco",
 "fa":"Farsi (Persa)",
 "fi":"Finlandês",
 "fj":"Fijiano",
 "fo":"Feroês",
 "fr":"Francês",
 "fy":"Frisão",
 "ga":"Gaélico irlandês",
 "gd":"Gaélico",
 "gl":"Galego",
 "gn":"Guarani",
 "gu":"Guzerate",
 "gv":"Manês",
 "ha":"Haussá",
 "he":"Hebraico",
 "hi":"Hindi",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Croata",
 "hsb":"Alto Sorábio",
 "hu":"Húngaro",
 "hy":"Armênio",
 "hz":"Hereró",
 "ia":"Interlíngua",
 "id":"Indonésio",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"Islandês",
 "it":"Italiano",
 "iu":"Inuktitut",
 "ja":"Japonês",
 "jv":"Javanês",
 "ka":"Georgiano",
 "ki":"Kikuyu",
 "kk":"Cazaque",
 "kl":"Groenlandês",
 "km":"Cambojano",
 "kn":"Canarês",
 "ko":"Coreano",
 "ks":"Caxemira",
 "ku":"Curdo",
 "kv":"Komi",
 "kw":"Córnico",
 "ky":"Quirguiz",
 "la":"Latim",
 "lb":"Luxemburguês",
 "li":"Limburguês",
 "ln":"Lingala",
 "lo":"Laociano",
 "lt":"Lituano",
 "lv":"Letão",
 "mai":"Maithili",
 "mg":"Malgaxe",
 "mh":"Marchalês",
 "mi":"Maori",
 "mk":"Macedônio",
 "ml":"Malaiala",
 "mn":"Mongol",
 "mo":"Moldavo",
 "mr":"Marata",
 "ms":"Malaio",
 "mt":"Maltês",
 "my":"Birmanês",
 "na":"Nauruano",
 "nb":"Dano-norueguês",
 "nd":"Ndebele do norte",
 "nds":"Baixo Saxão",
 "ne":"Nepalês",
 "ng":"Ndonga",
 "nl":"Holandês",
 "nn":"Novo norueguês",
 "nr":"Ndebele do Sul",
 "nso":"Sotho setentrional",
 "nv":"Navajo",
 "ny":"Cinianja",
 "oc":"Occitano",
 "om":"Oromo",
 "or":"Oriá",
 "os":"Osseto",
 "pa":"Panjabi",
 "pi":"Pali",
 "pl":"Polonês",
 "ps":"Afegão",
 "pt":"Português",
 "pt_BR":"Português do Brasil",
 "qu":"Quíchua",
 "rn":"Rundi",
 "ro":"Romeno",
 "rom":"Romani",
 "ru":"Russo",
 "rw":"Kinyarwanda",
 "sa":"Sânscrito",
 "sc":"Sardo",
 "sd":"Sindi",
 "se":"Sami setentrional",
 "sg":"Sango",
 "si":"Cingalês",
 "sk":"Eslovaco",
 "sl":"Esloveno",
 "sm":"Samoano",
 "sn":"Chona",
 "so":"Somali",
 "sq":"Albanês",
 "sr":"Sérvio",
 "sr@ijekavian":"Sérvio ijekavian",
 "sr@ijekavianlatin":"Sérvio latino ijekavian",
 "sr@latin":"Sérvio latino",
 "ss":"Suázi",
 "st":"Sotho meridional",
 "su":"Sudanês",
 "sv":"Sueco",
 "sw":"Suaíli",
 "ta":"Tâmil",
 "te":"Telugu",
 "tg":"Tajique",
 "th":"Tailandês",
 "ti":"Tigrino",
 "tk":"Turcomeno",
 "tn":"Tswana",
 "to":"Tonganês",
 "tr":"Turco",
 "ts":"Tsonga",
 "tt":"Tártaro",
 "tw":"Twi",
 "ty":"Taitiano",
 "ug":"Uigur",
 "uk":"Ucraniano",
 "ur":"Urdu",
 "uz":"Uzbeque",
 "uz@cyrillic":"Uzbeque (Cirílico)",
 "ven":"Venda",
 "vi":"Vietnamita",
 "vo":"Volapuque",
 "wa":"Valão",
 "wo":"Jalofo",
 "xh":"Xhosa",
 "yi":"Iídiche",
 "yo":"Iorubá",
 "za":"Zhuang",
 "zh":"Chinês",
 "zh_CN":"Chinês simplificado",
 "zh_HK":"Chinês (Hong Kong)",
 "zh_TW":"Chinês tradicional",
 "zu":"Zulu",
},
"ru": {
 "aa":"Афарский",
 "ab":"Абхазский",
 "ae":"Авестийский",
 "af":"Африкаанс",
 "am":"Амхарский",
 "ar":"Арабский",
 "as":"Ассамский",
 "ast":"Астурийский",
 "ay":"Аймарский",
 "az":"Азербайджанский",
 "ba":"Башкирский",
 "be":"Белорусский",
 "be@latin":"Белорусский (латиница)",
 "bg":"Болгарский",
 "bh":"Бихарский",
 "bi":"Бислама",
 "bn":"Бенгальский",
 "bn_IN":"Бенгальский (Индия)",
 "bo":"Тибетский",
 "br":"Бретонский",
 "bs":"Боснийский",
 "ca":"Каталонский",
 "ca@valencia":"Каталонский (Валенсия)",
 "ce":"Чеченский",
 "ch":"Чаморро",
 "co":"Корсиканский",
 "crh":"Крымско-татарский",
 "cs":"Чешский",
 "csb":"Кашубский",
 "cu":"Церковнославянский",
 "cv":"Чувашский",
 "cy":"Валлийский",
 "da":"Датский",
 "de":"Немецкий",
 "dsb":"Нижнелужицкий",
 "dz":"Дзонгка (Бутан)",
 "el":"Греческий",
 "en":"Английский",
 "en_GB":"Английский (Великобритания)",
 "en_US":"Английский (США)",
 "eo":"Эсперанто",
 "es":"Испанский",
 "et":"Эстонский",
 "eu":"Баскский",
 "fa":"Персидский",
 "fi":"Финский",
 "fj":"Фиджи",
 "fo":"Фарерский",
 "fr":"Французский",
 "fy":"Фризский",
 "ga":"Ирландский",
 "gd":"Шотландский",
 "gl":"Галисийский",
 "gn":"Гуарани",
 "gu":"Гуджарати",
 "gv":"Мэнский",
 "ha":"Хауса",
 "he":"Иврит",
 "hi":"Хинди",
 "hne":"Чхаттисгархи",
 "ho":"Хири-моту",
 "hr":"Хорватский",
 "hsb":"Верхнелужицкий",
 "hu":"Венгерский",
 "hy":"Армянский",
 "hz":"Эреро",
 "ia":"Интерлингва",
 "id":"Индонезийский",
 "ie":"Окциденталь",
 "ik":"Инупиак",
 "io":"Идо",
 "is":"Исландский",
 "it":"Итальянский",
 "iu":"Инуктитут",
 "ja":"Японский",
 "jv":"Яванский",
 "ka":"Грузинский",
 "ki":"Кикуйю",
 "kk":"Казахский",
 "kl":"Гренландский",
 "km":"Кхмерский",
 "kn":"Каннада",
 "ko":"Корейский",
 "ks":"Кашмирский",
 "ku":"Курдский",
 "kv":"Коми",
 "kw":"Корнский",
 "ky":"Киргизский",
 "la":"Латинский",
 "lb":"Люксембургский",
 "li":"Лимбуржский",
 "ln":"Лингала",
 "lo":"Лаосский",
 "lt":"Литовский",
 "lv":"Латышский",
 "mai":"Майтхили",
 "mg":"Мальгашский",
 "mh":"Маршалльский",
 "mi":"Маори",
 "mk":"Македонский",
 "ml":"Малаялам",
 "mn":"Монгольский",
 "mo":"Молдавский",
 "mr":"Маратхи",
 "ms":"Малайский",
 "mt":"Мальтийский",
 "my":"Бирманский",
 "na":"Науру",
 "nb":"Норвежский (литературный)",
 "nd":"Северный ндебеле",
 "nds":"Нижнесаксонский",
 "ne":"Непальский",
 "ng":"Ндонга",
 "nl":"Голландский",
 "nn":"Норвежский (нюнорск)",
 "nr":"Южный ндебеле",
 "nso":"Северный сото",
 "nv":"Навахо",
 "ny":"Чичева",
 "oc":"Окситанский",
 "om":"Оромо",
 "or":"Ория",
 "os":"Осетинский",
 "pa":"Панджаби",
 "pi":"Пали",
 "pl":"Польский",
 "ps":"Пушту",
 "pt":"Португальский",
 "pt_BR":"Португальский (Бразилия)",
 "qu":"Кечуа",
 "rn":"Рунди",
 "ro":"Румынский",
 "rom":"Цыганский",
 "ru":"Русский",
 "rw":"Киньяруанда",
 "sa":"Санскрит",
 "sc":"Сардинский",
 "sd":"Синдхи",
 "se":"Северносаамский",
 "sg":"Санго",
 "si":"Сингальский",
 "sk":"Словацкий",
 "sl":"Словенский",
 "sm":"Самоанский",
 "sn":"Шона",
 "so":"Сомалийский",
 "sq":"Албанский",
 "sr":"Сербский",
 "sr@ijekavian":"Сербский (иекавский)",
 "sr@ijekavianlatin":"Сербский (иекавский, латиница)",
 "sr@latin":"Сербский (латиница)",
 "ss":"Свати",
 "st":"Южный сото",
 "su":"Сунданский",
 "sv":"Шведский",
 "sw":"Суахили",
 "ta":"Тамильский",
 "te":"Телугу",
 "tg":"Таджикский",
 "th":"Тайский",
 "ti":"Тигринья",
 "tk":"Туркменский",
 "tn":"Тсвана",
 "to":"Тонганский",
 "tr":"Турецкий",
 "ts":"Тсонга",
 "tt":"Татарский",
 "tw":"Тви",
 "ty":"Таитянский",
 "ug":"Уйгурский",
 "uk":"Украинский",
 "ur":"Урду",
 "uz":"Узбекский",
 "uz@cyrillic":"Узбекский (кириллица)",
 "ven":"Венда",
 "vi":"Вьетнамский",
 "vo":"Волапюк",
 "wa":"Валлонский",
 "wo":"Волоф",
 "xh":"Коса",
 "yi":"Идиш",
 "yo":"Йоруба",
 "za":"Чжуанский",
 "zh":"Китайский",
 "zh_CN":"Китайский (Китай)",
 "zh_HK":"Китайский (Гонконг)",
 "zh_TW":"Китайский (Тайвань)",
 "zu":"Зулусский",
},
"tr": {
 "aa":"Afar",
 "ab":"Abkhazian",
 "ae":"Avestan",
 "af":"Afrika Dili",
 "am":"Amharic",
 "ar":"Arapça",
 "as":"Assamese",
 "ast":"Avusturya Dili",
 "ay":"Aymara",
 "az":"Azerbeycanca",
 "ba":"Bashkir",
 "be":"Belarus Dili",
 "be@latin":"Belarus Dili (Latin)",
 "bg":"Bulgarca",
 "bh":"Bihari",
 "bi":"Bislama",
 "bn":"Bengalce",
 "bn_IN":"Bengalce (Hindistan)",
 "bo":"Tibetan",
 "br":"Britanya Dili",
 "bs":"Boşnakça",
 "ca":"Katalan Dili",
 "ca@valencia":"Katalan (Valencia)",
 "ce":"Çeçen",
 "ch":"Chamorro",
 "co":"Korsikaca",
 "crh":"Kırım Tatarcası",
 "cs":"Çekçe",
 "csb":"Kashubian",
 "cu":"Church Slavic",
 "cv":"Çuvaşça",
 "cy":"Gallerce",
 "da":"Danimarka Dili",
 "de":"Almanca",
 "dsb":"Aşağı Sırpça",
 "dz":"Dzongkha",
 "el":"Yunanca",
 "en":"İngilizce",
 "en_GB":"İngiliz İngilizcesi",
 "en_US":"Amerikan İngilizcesi",
 "eo":"Esperanto",
 "es":"İspanyolca",
 "et":"Estonya Dili",
 "eu":"Bask Dili",
 "fa":"Farsça (İran)",
 "fi":"Fince",
 "fj":"Fijian",
 "fo":"Faroese",
 "fr":"Fransızca",
 "fy":"Frisian",
 "ga":"İrlanda Galik",
 "gd":"Galik",
 "gl":"Galce",
 "gn":"Guarani",
 "gu":"Gujarati",
 "gv":"Manx",
 "ha":"Hausa",
 "he":"İbranice",
 "hi":"Hintçe",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu",
 "hr":"Hırvatça",
 "hsb":"Yukarı Sırpça",
 "hu":"Macarca",
 "hy":"Ermenice",
 "hz":"Herero",
 "ia":"Interlingua",
 "id":"İndonezya Dili",
 "ie":"Interlingue",
 "ik":"Inupiaq",
 "io":"Ido",
 "is":"İzlanda Dili",
 "it":"İtalyanca",
 "iu":"Inuktitut",
 "ja":"Japonca",
 "jv":"Javanese",
 "ka":"Gürcüce",
 "ki":"Kikuyu",
 "kk":"Kazakça",
 "kl":"Kalaallisut",
 "km":"Khmer",
 "kn":"Kannada",
 "ko":"Korece",
 "ks":"Kashmiri",
 "ku":"Kürtçe",
 "kv":"Komi",
 "kw":"Cornish",
 "ky":"Kırgız",
 "la":"Latin",
 "lb":"Luxembourgish",
 "li":"Limburgan",
 "ln":"Lingala",
 "lo":"Lao",
 "lt":"Litvanya Dili",
 "lv":"Letonca",
 "mai":"Maithili",
 "mg":"Malagasy",
 "mh":"Marshallese",
 "mi":"Maori Dili",
 "mk":"Makedonca",
 "ml":"Malayalam",
 "mn":"Moğolca",
 "mo":"Moldavya Dili",
 "mr":"Marathi",
 "ms":"Malay",
 "mt":"Malta Dili",
 "my":"Burmese",
 "na":"Nauru",
 "nb":"Norveççe Bokmål",
 "nd":"Ndebele, North",
 "nds":"Aşağı Saksonca",
 "ne":"Nepalce",
 "ng":"Ndonga",
 "nl":"Flamanca",
 "nn":"Norveççe Nynorsk",
 "nr":"Ndebele, South",
 "nso":"Northern Sotho",
 "nv":"Navajo",
 "ny":"Chichewa",
 "oc":"Oksitan dili",
 "om":"Oromo",
 "or":"Oriya",
 "os":"Ossetian",
 "pa":"Punjabi/Panjabi",
 "pi":"Pali",
 "pl":"Lehçe",
 "ps":"Pushto",
 "pt":"Portekizce",
 "pt_BR":"Brezilya Portekizcesi",
 "qu":"Quechua",
 "rn":"Rundi",
 "ro":"Romence",
 "rom":"Çingene Dili",
 "ru":"Rusça",
 "rw":"Kinyarwanda",
 "sa":"Sanskrit",
 "sc":"Sardinian",
 "sd":"Sindhi",
 "se":"Northern Sami",
 "sg":"Sango",
 "si":"Sinhala",
 "sk":"Slovakça",
 "sl":"Slovence",
 "sm":"Samoan",
 "sn":"Shona",
 "so":"Somalice",
 "sq":"Albanian",
 "sr":"Sırpça",
 "sr@ijekavian":"Sırpça Ijekavian",
 "sr@ijekavianlatin":"Sırpça Ijekavian Latin",
 "sr@latin":"Sırpça Latin",
 "ss":"Swati",
 "st":"Sotho, Southern",
 "su":"Sundanese",
 "sv":"İsveççe",
 "sw":"Swahili",
 "ta":"Tamil Dili",
 "te":"Telugu",
 "tg":"Tajik",
 "th":"Thai",
 "ti":"Tigrinya",
 "tk":"Türkmence",
 "tn":"Tswana",
 "to":"Tonga",
 "tr":"Türkçe",
 "ts":"Tsonga",
 "tt":"Tatar",
 "tw":"Twi",
 "ty":"Tahitian",
 "ug":"Uygurca",
 "uk":"Ukrayna Dili",
 "ur":"Urdu",
 "uz":"Özbek",
 "uz@cyrillic":"Özbekçe (Kiril)",
 "ven":"Venda",
 "vi":"Vietnamca",
 "vo":"Volapük",
 "wa":"Valonca",
 "wo":"Wolof",
 "xh":"Xhosa",
 "yi":"Yiddish",
 "yo":"Yoruba",
 "za":"Zhuang",
 "zh":"Çince",
 "zh_CN":"Basitleştirilmiş Çince",
 "zh_HK":"Çince (Hong Kong)",
 "zh_TW":"Geleneksel Çince",
 "zu":"Zulu",
},
"uk": {
 "aa":"Афар",
 "ab":"Абхазька",
 "ae":"Авестан",
 "af":"Африкаанс",
 "am":"Амхарська",
 "ar":"Арабська",
 "as":"Асамійська",
 "ast":"Астурійська",
 "ay":"Аймарська",
 "az":"Азербайджанська",
 "ba":"Башкирська",
 "be":"Білоруська",
 "be@latin":"Білоруська (латиниця)",
 "bg":"Болгарська",
 "bh":"Біхарі",
 "bi":"Біслама",
 "bn":"Бенгальська",
 "bn_IN":"Бенгальська (Індія)",
 "bo":"Тибетська",
 "br":"Бретонська",
 "bs":"Боснійська",
 "ca":"Каталонська",
 "ca@valencia":"Каталонська (валенсійський діалект)",
 "ce":"Чеченська",
 "ch":"Чаморо",
 "co":"Корсиканська",
 "crh":"Кримсько-татарська",
 "cs":"Чеська",
 "csb":"Кашубська",
 "cu":"Церковнослов'янська",
 "cv":"Чуваська",
 "cy":"Уельська",
 "da":"Данська",
 "de":"Німецька",
 "dsb":"Нижньолужицька",
 "dz":"Дзонг-ке",
 "el":"Грецька",
 "en":"Англійська",
 "en_GB":"Англійська (Великобританія)",
 "en_US":"Англійська (США)",
 "eo":"Есперанто",
 "es":"Іспанська",
 "et":"Естонська",
 "eu":"Баскська",
 "fa":"Фарсі (Перська)",
 "fi":"Фінська",
 "fj":"Фіджійська",
 "fo":"Фарерська",
 "fr":"Французька",
 "fy":"Фризька",
 "ga":"Гаельська (Ірландія)",
 "gd":"Гаельська",
 "gl":"Галісійська",
 "gn":"Гуарані",
 "gu":"Гуджараті",
 "gv":"Манкс",
 "ha":"Гауса",
 "he":"Єврейська",
 "hi":"Хінді",
 "hne":"Чхатісгарі",
 "ho":"Хірімоту",
 "hr":"Хорватська",
 "hsb":"Верхньолужицька",
 "hu":"Угорська",
 "hy":"Вірменська",
 "hz":"Гереро",
 "ia":"Інтерлінгва",
 "id":"Індонезійська",
 "ie":"Окциденталь",
 "ik":"Інупіак",
 "io":"Ідо",
 "is":"Ісландська",
 "it":"Італійська",
 "iu":"Інуктітут",
 "ja":"Японська",
 "jv":"Яванська",
 "ka":"Грузинська",
 "ki":"Кікуйю",
 "kk":"Казахська",
 "kl":"Калаалісут",
 "km":"Кхмерська",
 "kn":"Каннада",
 "ko":"Корейська",
 "ks":"Кашмірська",
 "ku":"Курдська",
 "kv":"Комі",
 "kw":"Корнійська",
 "ky":"Киргизька",
 "la":"Латинська",
 "lb":"Люксембурзька",
 "li":"Лімбурганська",
 "ln":"Лінгала",
 "lo":"Лаоська",
 "lt":"Литовська",
 "lv":"Латвійська",
 "mai":"Майтхілі",
 "mg":"Малагасійська",
 "mh":"Маршальська",
 "mi":"Маорі",
 "mk":"Македонська",
 "ml":"Малайська",
 "mn":"Монгольська",
 "mo":"Молдовська",
 "mr":"Мараті",
 "ms":"Малайська",
 "mt":"Мальтійська",
 "my":"Бірманська",
 "na":"Науру",
 "nb":"Норвезька (Bokmål)",
 "nd":"Ндебелє, Північна",
 "nds":"Нижньосаксонська",
 "ne":"Непальська",
 "ng":"Ндонга",
 "nl":"Голландська",
 "nn":"Норвезька (Nynorsk)",
 "nr":"Ндебелє, Південна",
 "nso":"Північне Сото",
 "nv":"Навахо",
 "ny":"Чічеванська",
 "oc":"Оксітанська",
 "om":"Оромо",
 "or":"Орія",
 "os":"Осетинська",
 "pa":"Панджабська",
 "pi":"Палі",
 "pl":"Польська",
 "ps":"Пуштунська",
 "pt":"Португальська",
 "pt_BR":"Бразильська португальська",
 "qu":"Кечуа",
 "rn":"Рунді",
 "ro":"Румунська",
 "rom":"Циганська",
 "ru":"Російська",
 "rw":"Кіньяруанда",
 "sa":"Санскрит",
 "sc":"Сардинська",
 "sd":"Синдхі",
 "se":"Північна Саамі",
 "sg":"Санго",
 "si":"Сингала",
 "sk":"Словацька",
 "sl":"Словенська",
 "sm":"Самоанська",
 "sn":"Шона",
 "so":"Сомалійська",
 "sq":"Албанська",
 "sr":"Сербська",
 "sr@ijekavian":"Сербська (ієкавиця)",
 "sr@ijekavianlatin":"Сербська (ієкавиця, латиниця)",
 "sr@latin":"Сербська (латиниця)",
 "ss":"Свазі",
 "st":"Сото, Південна",
 "su":"Сунданська",
 "sv":"Шведська",
 "sw":"Суахілі",
 "ta":"Тамільська",
 "te":"Телугу",
 "tg":"Таджицька",
 "th":"Тайська",
 "ti":"Тигринійська",
 "tk":"Туркменська",
 "tn":"Тсвана",
 "to":"Тонга",
 "tr":"Турецька",
 "ts":"Цонґа",
 "tt":"Татарська",
 "tw":"Тві",
 "ty":"Таїтянська",
 "ug":"Уйгурська",
 "uk":"Українська",
 "ur":"Урду",
 "uz":"Узбецька",
 "uz@cyrillic":"Узбецька (кирилиця)",
 "ven":"Венда",
 "vi":"В'єтнамська",
 "vo":"Волапюк",
 "wa":"Валлонська",
 "wo":"Волоф",
 "xh":"Хоза",
 "yi":"Ідиш",
 "yo":"Йоруба",
 "za":"Чжуань",
 "zh":"Китайська",
 "zh_CN":"Китайська (спрощена)",
 "zh_HK":"Китайська (Гонконг)",
 "zh_TW":"Китайська (традиційна)",
 "zu":"Зулуська",
},
"zh_CN": {
 "aa":"阿法尔语",
 "ab":"阿布哈西亚语",
 "ae":"阿维斯陀语",
 "af":"南非荷兰语",
 "am":"阿比西尼亚语",
 "ar":"阿拉伯语",
 "as":"阿萨姆语",
 "ast":"阿斯图里亚斯语",
 "ay":"艾马拉语",
 "az":"阿塞拜疆语",
 "ba":"巴什基尔语",
 "be":"白俄罗斯语",
 "be@latin":"白俄罗斯语(拉丁语系)",
 "bg":"保加利亚语",
 "bh":"比哈尔语",
 "bi":"比斯拉马语",
 "bn":"孟加拉语",
 "bn_IN":"孟加拉语(印度)",
 "bo":"藏语",
 "br":"布里多尼语",
 "bs":"波斯尼亚语",
 "ca":"加泰罗尼亚语",
 "ca@valencia":"加泰罗尼亚(瓦伦西亚)语",
 "ce":"车臣语",
 "ch":"查莫罗语",
 "co":"科西嘉语",
 "crh":"克里米亚鞑靼语",
 "cs":"捷克语",
 "csb":"卡舒比语",
 "cu":"古教会斯拉夫语",
 "cv":"楚瓦什语",
 "cy":"威尔士语",
 "da":"丹麦语",
 "de":"德语",
 "dsb":"下索布语",
 "dz":"不丹语",
 "el":"希腊语",
 "en":"英语",
 "en_GB":"英国英语",
 "en_US":"美国英语",
 "eo":"世界语",
 "es":"西班牙语",
 "et":"爱沙尼亚语",
 "eu":"巴斯克语",
 "fa":"波斯语",
 "fi":"芬兰语",
 "fj":"斐济语",
 "fo":"法罗群岛语",
 "fr":"法语",
 "fy":"弗里斯兰语",
 "ga":"爱尔兰盖尔语",
 "gd":"盖尔语",
 "gl":"加利西亚语",
 "gn":"瓜拉尼语",
 "gu":"古吉拉特语",
 "gv":"马恩语",
 "ha":"豪撒语",
 "he":"希伯来语",
 "hi":"印度语",
 "hne":"恰蒂斯加尔语",
 "ho":"希里莫图语",
 "hr":"克罗地亚语",
 "hsb":"上索布语",
 "hu":"匈牙利语",
 "hy":"亚美尼亚语",
 "hz":"赫雷罗语",
 "ia":"拉丁国际语",
 "id":"印度尼西亚语",
 "ie":"国际语",
 "ik":"因纽皮特语",
 "io":"伊多语",
 "is":"冰岛语",
 "it":"意大利语",
 "iu":"因纽特语",
 "ja":"日语",
 "jv":"爪哇语",
 "ka":"乔治亚语",
 "ki":"基库尤语",
 "kk":"哈萨克语",
 "kl":"格陵兰语",
 "km":"高棉语",
 "kn":"卡纳达语",
 "ko":"韩语",
 "ks":"克什米尔语",
 "ku":"库尔德语",
 "kv":"科米语",
 "kw":"康沃尔语",
 "ky":"吉尔吉斯语",
 "la":"拉丁语",
 "lb":"卢森堡语",
 "li":"林堡语",
 "ln":"林加拉语",
 "lo":"老挝语",
 "lt":"立陶宛语",
 "lv":"拉脱维亚语",
 "mai":"迈蒂利语",
 "mg":"马尔加什语",
 "mh":"马绍尔语",
 "mi":"毛利语",
 "mk":"马其顿语",
 "ml":"马拉亚拉姆语",
 "mn":"蒙古语",
 "mo":"摩尔多瓦语",
 "mr":"马拉提语",
 "ms":"马来语",
 "mt":"马耳他语",
 "my":"缅甸语",
 "na":"瑙鲁语",
 "nb":"挪威语(博克马尔语)",
 "nd":"恩德贝莱语(北部)",
 "nds":"下撒克逊语",
 "ne":"尼泊尔语",
 "ng":"恩东加语",
 "nl":"荷兰语",
 "nn":"挪威语(尼诺斯克语)",
 "nr":"恩德贝莱语(南部)",
 "nso":"北索托语",
 "nv":"纳瓦霍语",
 "ny":"齐佩瓦语",
 "oc":"欧西坦语",
 "om":"奥罗莫语",
 "or":"奥利亚语",
 "os":"奥塞梯语",
 "pa":"旁遮普语",
 "pi":"巴利语",
 "pl":"波兰语",
 "ps":"普什图语",
 "pt":"葡萄牙语",
 "pt_BR":"巴西葡萄牙语",
 "qu":"盖丘亚语",
 "rn":"隆迪语",
 "ro":"罗马尼亚语",
 "rom":"吉普赛语",
 "ru":"俄语",
 "rw":"基尼亚卢旺达语",
 "sa":"梵文",
 "sc":"撒丁语",
 "sd":"信德语",
 "se":"北萨米语",
 "sg":"桑戈语",
 "si":"僧伽罗语",
 "sk":"斯洛伐克语",
 "sl":"斯洛文尼亚语",
 "sm":"萨摩亚语",
 "sn":"修纳语",
 "so":"索马里语",
 "sq":"阿尔巴尼亚语",
 "sr":"塞尔维亚语",
 "sr@ijekavian":"塞尔维亚 Ijekavian 口音语言",
 "sr@ijekavianlatin":"塞尔维亚 Ijekavian 口音拉丁语",
 "sr@latin":"塞尔维亚拉丁语",
 "ss":"斯瓦蒂语",
 "st":"索托语(南部)",
 "su":"巽他语",
 "sv":"瑞典语",
 "sw":"斯瓦希里语",
 "ta":"泰米尔语",
 "te":"泰卢固语",
 "tg":"塔吉克语",
 "th":"泰语",
 "ti":"提格里尼亚语",
 "tk":"土库曼语",
 "tn":"茨瓦纳语",
 "to":"汤加语",
 "tr":"土耳其语",
 "ts":"聪加语",
 "tt":"鞑靼语",
 "tw":"特威语",
 "ty":"塔希提语",
 "ug":"维吾尔语",
 "uk":"乌克兰语",
 "ur":"乌尔都语",
 "uz":"乌兹别克语",
 "uz@cyrillic":"乌兹别克语(西里尔语系)",
 "ven":"文达语",
 "vi":"越南语",
 "vo":"沃拉普克语",
 "wa":"瓦龙语",
 "wo":"沃洛夫语",
 "xh":"科萨语",
 "yi":"依地语",
 "yo":"约鲁巴语",
 "za":"壮语",
 "zh":"中文",
 "zh_CN":"简体中文",
 "zh_HK":"繁体中文(香港)",
 "zh_TW":"繁体中文",
 "zu":"祖鲁语",
},
"zh_HK": {
 "aa":"阿發爾",
 "ab":"阿布哈西亞語",
 "ae":"阿維斯陀語",
 "af":"南非荷蘭語",
 "am":"衣索比亞官方語",
 "ar":"阿拉伯語",
 "as":"阿薩姆語",
 "ay":"愛瑪拉語",
 "az":"亞塞拜彊語",
 "ba":"Bashkir語",
 "be":"白俄羅斯語",
 "bg":"保加利亞語",
 "bh":"比哈爾語",
 "bi":"Bislama語",
 "bn":"孟加拉語",
 "bo":"藏語",
 "br":"不列塔尼語",
 "bs":"波斯尼亞語",
 "ca":"嘉泰羅尼亞語",
 "ce":"Chechen語",
 "ch":"查摩洛語",
 "co":"科西嘉語",
 "cs":"捷克語",
 "cu":"教堂斯拉夫語",
 "cv":"楚瓦士語",
 "cy":"威爾斯語",
 "da":"丹麥語",
 "de":"德語",
 "dz":"Dzongkha語",
 "el":"希臘語",
 "en":"英語",
 "en_GB":"英式英語",
 "en_US":"美式英語",
 "eo":"世界語",
 "es":"西班牙語",
 "et":"愛沙尼亞語",
 "eu":"巴斯克語",
 "fa":"波斯語",
 "fi":"芬蘭語",
 "fj":"斐濟語",
 "fo":"法羅語",
 "fr":"法語",
 "fy":"弗利然語",
 "gd":"蓋爾語",
 "gl":"加利西亞語",
 "gn":"瓜拉尼語",
 "gu":"古吉拉特語",
 "gv":"曼島語",
 "ha":"豪薩語",
 "he":"希伯來語",
 "hi":"北印度語",
 "ho":"Hiri Motu語",
 "hr":"克羅地亞語",
 "hu":"匈牙利語",
 "hy":"亞美尼亞語",
 "hz":"Herero語",
 "ia":"科技共通語",
 "id":"印尼語",
 "ie":"Interlingue語",
 "ik":"Inupiaq語",
 "io":"伊多語",
 "is":"冰島語",
 "it":"意大利語",
 "iu":"Inuktitut語",
 "ja":"日語",
 "jv":"爪哇語",
 "ka":"喬治亞語",
 "ki":"吉庫猶語",
 "kk":"哈薩克語",
 "kl":"Kalaallisut語",
 "km":"高棉語",
 "kn":"坎那達語",
 "ko":"韓國語",
 "ks":"喀什米爾語",
 "ku":"庫德語",
 "kv":"Komi語",
 "kw":"康瓦耳語",
 "ky":"吉爾吉斯語",
 "la":"拉丁語",
 "lb":"盧森堡語",
 "li":"Limburgan語",
 "ln":"Lingala語",
 "lo":"寮國語",
 "lt":"立陶宛語",
 "lv":"拉脫維亞語",
 "mg":"馬拉加西語",
 "mh":"馬紹爾語",
 "mi":"毛利語",
 "mk":"馬其頓語",
 "ml":"馬來亞拉姆語",
 "mn":"蒙古語",
 "mo":"摩爾達維亞語",
 "mr":"馬拉地語",
 "ms":"馬來語",
 "mt":"馬爾他語",
 "my":"緬甸語",
 "na":"諾魯語",
 "nd":"Ndebele語，北部",
 "ne":"尼泊爾語",
 "ng":"Ndonga語",
 "nl":"荷蘭語",
 "nn":"挪威 Nynorsk 語",
 "nr":"Ndebele語，南部",
 "nv":"納瓦伙語",
 "ny":"Chichewa語",
 "oc":"Occitan語",
 "om":"Oromo語",
 "or":"Oriya語",
 "os":"Ossetian語",
 "pi":"巴利語",
 "pl":"波蘭語",
 "ps":"普什圖語",
 "pt":"葡萄牙語",
 "pt_BR":"巴西葡萄牙語",
 "qu":"蓋楚瓦語",
 "rn":"Rundi語",
 "ro":"羅馬尼亞語",
 "ru":"俄語",
 "rw":"Kinyarwanda語",
 "sa":"梵語",
 "sc":"薩丁尼亞語",
 "sd":"Sindhi語",
 "se":"北薩米語",
 "sg":"Sango語",
 "sk":"斯洛伐克語",
 "sl":"斯洛維尼亞語",
 "sm":"薩摩牙語",
 "sn":"Shona語",
 "so":"索馬利語",
 "sq":"阿爾巴尼亞語",
 "sr":"塞爾維亞語",
 "ss":"Swati語",
 "st":"梭托語，南部",
 "su":"Sundanese語",
 "sv":"瑞典語",
 "sw":"斯華西里語",
 "ta":"坦米爾語",
 "te":"特拉古語",
 "tg":"塔吉克語",
 "th":"泰國語",
 "ti":"提格利尼亞語",
 "tk":"土庫曼語",
 "tn":"班圖語",
 "to":"東加語",
 "tr":"土耳其語",
 "ts":"Tsonga語",
 "tt":"韃靼語",
 "tw":"契維語",
 "ty":"大溪地語",
 "uk":"烏克蘭語",
 "ur":"烏都語",
 "uz":"烏茲別克語",
 "vi":"越南語",
 "wa":"華隆語",
 "wo":"Wolof語",
 "xh":"科薩語",
 "yi":"意第緒語",
 "yo":"優魯巴語",
 "za":"Zhuang語",
 "zh":"中文",
 "zh_CN":"簡體中文",
 "zh_HK":"繁體中文（香港）",
 "zh_TW":"繁體中文",
 "zu":"袓魯語",
},
"zh_TW": {
 "aa":"阿發爾",
 "ab":"阿布哈西亞語",
 "ae":"阿維斯陀語",
 "af":"南非荷蘭語",
 "am":"衣索比亞官方語",
 "ar":"阿拉伯語",
 "as":"阿薩姆語",
 "ast":"Asturian",
 "ay":"愛瑪拉語",
 "az":"亞塞拜然語",
 "ba":"Bashkir語",
 "be":"白俄羅斯語",
 "be@latin":"白俄羅斯語（拉丁）",
 "bg":"保加利亞語",
 "bh":"比哈爾語",
 "bi":"Bislama語",
 "bn":"孟加拉語",
 "bn_IN":"孟加拉語（印度）",
 "bo":"藏語",
 "br":"不列塔尼語",
 "bs":"波士尼亞語",
 "ca":"嘉泰羅尼亞語",
 "ca@valencia":"加泰羅尼亞（瓦倫西亞）語",
 "ce":"車臣語",
 "ch":"查摩洛語",
 "co":"科西嘉語",
 "crh":"Crimean Tatar",
 "cs":"捷克語",
 "csb":"Kashubian",
 "cu":"教堂斯拉夫語",
 "cv":"楚瓦士語",
 "cy":"威爾斯語",
 "da":"丹麥語",
 "de":"德語",
 "dsb":"下塞爾維亞語",
 "dz":"Dzongkha語",
 "el":"希臘語",
 "en":"英語",
 "en_GB":"（英式）英語",
 "en_US":"美語",
 "eo":"世界語",
 "es":"西班牙語",
 "et":"愛沙尼亞語",
 "eu":"巴斯克語",
 "fa":"波斯語",
 "fi":"芬蘭語",
 "fj":"斐濟語",
 "fo":"法羅語",
 "fr":"法語",
 "fy":"弗利然語",
 "ga":"（愛爾蘭）蓋爾語",
 "gd":"蓋爾語",
 "gl":"加利西亞語",
 "gn":"瓜拉尼語",
 "gu":"古吉拉特語",
 "gv":"曼島語",
 "ha":"豪薩語",
 "he":"希伯來語",
 "hi":"北印度語",
 "hne":"Chhattisgarhi",
 "ho":"Hiri Motu語",
 "hr":"克羅埃西亞語",
 "hsb":"塞爾維亞語",
 "hu":"匈牙利語",
 "hy":"亞美尼亞語",
 "hz":"Herero語",
 "ia":"科技共通語",
 "id":"印尼語",
 "ie":"Interlingue語",
 "ik":"Inupiaq語",
 "io":"伊多語",
 "is":"冰島語",
 "it":"義大利語",
 "iu":"因紐特語",
 "ja":"日語",
 "jv":"爪哇語",
 "ka":"喬治亞語",
 "ki":"吉庫猶語",
 "kk":"哈薩克語",
 "kl":"Kalaallisut語",
 "km":"高棉語",
 "kn":"坎那達語",
 "ko":"韓國語",
 "ks":"喀什米爾語",
 "ku":"庫德語",
 "kv":"Komi語",
 "kw":"康瓦耳語",
 "ky":"吉爾吉斯語",
 "la":"拉丁語",
 "lb":"盧森堡語",
 "li":"Limburgan語",
 "ln":"Lingala語",
 "lo":"寮國語",
 "lt":"立陶宛語",
 "lv":"拉脫維亞語",
 "mai":"Maithili語",
 "mg":"馬拉加西語",
 "mh":"馬紹爾語",
 "mi":"毛利語",
 "mk":"馬其頓語",
 "ml":"馬來亞拉姆語",
 "mn":"蒙古語",
 "mo":"摩爾達維亞語",
 "mr":"馬拉地語",
 "ms":"馬來語",
 "mt":"馬爾他語",
 "my":"緬甸語",
 "na":"諾魯語",
 "nb":"挪威 Bokmål",
 "nd":"Ndebele語，北部",
 "nds":"薩克遜語",
 "ne":"尼泊爾語",
 "ng":"Ndonga語",
 "nl":"荷蘭語",
 "nn":"挪威 Nynorsk 語",
 "nr":"Ndebele語，南部",
 "nso":"Northern Sotho",
 "nv":"納瓦伙語",
 "ny":"Chichewa語",
 "oc":"Occitan語",
 "om":"Oromo語",
 "or":"Oriya語",
 "os":"Ossetian語",
 "pa":"Punjabi/Panjabi",
 "pi":"巴利語",
 "pl":"波蘭語",
 "ps":"普什圖語",
 "pt":"葡萄牙語",
 "pt_BR":"巴西葡萄牙語",
 "qu":"蓋楚瓦語",
 "rn":"Rundi語",
 "ro":"羅馬尼亞語",
 "rom":"吉普賽語",
 "ru":"俄語",
 "rw":"Kinyarwanda語",
 "sa":"梵語",
 "sc":"薩丁尼亞語",
 "sd":"Sindhi語",
 "se":"北薩米語",
 "sg":"Sango語",
 "si":"Sinhala",
 "sk":"斯洛伐克語",
 "sl":"斯洛維尼亞語",
 "sm":"薩摩亞語",
 "sn":"Shona語",
 "so":"索馬利語",
 "sq":"阿爾巴尼亞語",
 "sr":"塞爾維亞語",
 "sr@ijekavian":"賽爾維亞耶卡語",
 "sr@ijekavianlatin":"賽爾維亞耶卡拉丁語",
 "sr@latin":"賽爾維亞拉丁語",
 "ss":"Swati語",
 "st":"梭托語，南部",
 "su":"Sundanese語",
 "sv":"瑞典語",
 "sw":"斯華西里語",
 "ta":"坦米爾語",
 "te":"特拉古語",
 "tg":"塔吉克語",
 "th":"泰國語",
 "ti":"提格利尼亞語",
 "tk":"土庫曼語",
 "tn":"班圖語",
 "to":"東加語",
 "tr":"土耳其語",
 "ts":"Tsonga語",
 "tt":"韃靼語",
 "tw":"契維語",
 "ty":"大溪地語",
 "ug":"Uyghur",
 "uk":"烏克蘭語",
 "ur":"烏都語",
 "uz":"烏茲別克語",
 "uz@cyrillic":"烏茲別克（斯拉夫）",
 "ven":"Venda",
 "vi":"越南語",
 "vo":"Volapük",
 "wa":"華隆語",
 "wo":"Wolof語",
 "xh":"科薩語",
 "yi":"意第緒語",
 "yo":"優魯巴語",
 "za":"Zhuang語",
 "zh":"中文",
 "zh_CN":"簡體中文",
 "zh_HK":"正體中文（香港）",
 "zh_TW":"正體中文",
 "zu":"袓魯語",
},
}

# End of data.py
